@@ -34,6 +34,7 @@ package com.vividsolutions.jts.algorithm;
 
 import com.vividsolutions.jts.geom.Coordinate;
 import com.vividsolutions.jts.geom.CoordinateSequence;
+import com.vividsolutions.jts.geom.Location;
 
 /**
  * Specifies and implements various fundamental Computational Geometric algorithms.
@@ -88,64 +89,47 @@ public class CGAlgorithms
   }
 
   /**
-   * Test whether a point lies inside a ring.
+   * Tests whether a point lies inside or on a ring.
    * The ring may be oriented in either direction.
-   * If the point lies on the ring boundary the result of this method is unspecified.
+   * A point lying exactly on the ring boundary is considered to be inside the ring.
    * <p>
-   * This algorithm does not attempt to first check the point against the envelope
+   * This method does <i>not</i> first check the point against the envelope
    * of the ring.
    *
    * @param p point to check for ring inclusion
-   * @param ring assumed to have first point identical to last point
-   * @return <code>true</code> if p is inside ring
+   * @param ring an array of coordinates representing the ring (which must have first point identical to last point)
+   * @return true if p is inside ring
+   * 
+   * @see locatePointInRing
    */
   public static boolean isPointInRing(Coordinate p, Coordinate[] ring) {
-    /*
-     *  For each segment l = (i-1, i), see if it crosses ray from test point in positive x direction.
-     */
-    int crossings = 0;  // number of segment/ray crossings
-    for (int i = 1; i < ring.length; i++) {
-      int i1 = i - 1;
-      Coordinate p1 = ring[i];
-      Coordinate p2 = ring[i1];
-
-      if (((p1.y > p.y) && (p2.y <= p.y)) ||
-          ((p2.y > p.y) && (p1.y <= p.y))) {
-        double x1 = p1.x - p.x;
-        double y1 = p1.y - p.y;
-        double x2 = p2.x - p.x;
-        double y2 = p2.y - p.y;
-        /*
-        *  segment straddles x axis, so compute intersection with x-axis.
-         */
-        double xInt = RobustDeterminant.signOfDet2x2(x1, y1, x2, y2) / (y2 - y1);
-        //xsave = xInt;
-        /*
-        *  crosses ray if strictly positive intersection.
-         */
-        if (xInt > 0.0) {
-          crossings++;
-        }
-      }
-    }
-    /*
-     *  p is inside if number of crossings is odd.
-     */
-    if ((crossings % 2) == 1) {
-      return true;
-    }
-    else {
-      return false;
-    }
+  	return locatePointInRing(p, ring) != Location.EXTERIOR;
   }
+  
 
   /**
-   * Test whether a point lies on the line segments defined by a
+   * Determines whether a point lies in the interior, on the boundary, or in the exterior
+   * of a ring.
+   * The ring may be oriented in either direction.
+   * <p>
+   * This method does <i>not</i> first check the point against the envelope
+   * of the ring.
+   *
+   * @param p point to check for ring inclusion
+   * @param ring an array of coordinates representing the ring (which must have first point identical to last point)
+   * @return the {@link Location} of p relative to the ring
+   */
+  public static int locatePointInRing(Coordinate p, Coordinate[] ring) 
+  {
+    return RayCrossingCounter.locatePointInRing(p, ring);
+  }
+  
+  /**
+   * Tests whether a point lies on the line segments defined by a
    * list of coordinates.
    *
-   * @return true true if
-   * the point is a vertex of the line or lies in the interior of a line
-   * segment in the linestring
+   * @return true if the point is a vertex of the line 
+   * or lies in the interior of a line segment in the linestring
    */
   public static boolean isOnLine(Coordinate p, Coordinate[] pt) {
     LineIntersector lineIntersector = new RobustLineIntersector();
@@ -161,44 +145,50 @@ public class CGAlgorithms
   }
 
   /**
-   * Computes whether a ring defined by an array of {@link Coordinate} is
+   * Computes whether a ring defined by an array of {@link Coordinate}s is
    * oriented counter-clockwise.
    * <ul>
    * <li>The list of points is assumed to have the first and last points equal.
    * <li>This will handle coordinate lists which contain repeated points.
-   * <li>If the ring is invalid, the answer returned may not be correct.
    * </ul>
+   * This algorithm is <b>only</b> guaranteed to work with valid rings.
+   * If the ring is invalid (e.g. self-crosses or touches),
+   * the computed result may not be correct.
    *
-   * @param ring an array of coordinates forming a ring
-   * @return <code>true</code> if the ring is oriented counter-clockwise.
+   * @param ring an array of Coordinates forming a ring
+   * @return true if the ring is oriented counter-clockwise.
+   * @throws IllegalArgumentException if there are too few points to determine orientation (< 3)
    */
   public static boolean isCCW(Coordinate[] ring) {
     // # of points without closing endpoint
     int nPts = ring.length - 1;
-
+    // sanity check
+    if (nPts < 3)
+    	throw new IllegalArgumentException("Ring has fewer than 3 points, so orientation cannot be determined");
+    
     // find highest point
-    Coordinate hip = ring[0];
-    int hii = 0;
+    Coordinate hiPt = ring[0];
+    int hiIndex = 0;
     for (int i = 1; i <= nPts; i++) {
       Coordinate p = ring[i];
-      if (p.y > hip.y) {
-        hip = p;
-        hii = i;
+      if (p.y > hiPt.y) {
+        hiPt = p;
+        hiIndex = i;
       }
     }
 
     // find distinct point before highest point
-    int iPrev = hii;
+    int iPrev = hiIndex;
     do {
       iPrev = iPrev - 1;
       if (iPrev < 0) iPrev = nPts;
-    } while (ring[iPrev].equals(hip) && iPrev != hii);
+    } while (ring[iPrev].equals2D(hiPt) && iPrev != hiIndex);
 
     // find distinct point after highest point
-    int iNext = hii;
+    int iNext = hiIndex;
     do {
       iNext = (iNext + 1) % nPts;
-    } while (ring[iNext].equals(hip) && iNext != hii);
+    } while (ring[iNext].equals2D(hiPt) && iNext != hiIndex);
 
     Coordinate prev = ring[iPrev];
     Coordinate next = ring[iNext];
@@ -209,12 +199,10 @@ public class CGAlgorithms
      * (including the case where the input array has fewer than 4 elements),
      * or it contains coincident line segments.
      */
-    if (prev.equals(hip) || next.equals(hip) || prev.equals(next))
+    if (prev.equals2D(hiPt) || next.equals2D(hiPt) || prev.equals2D(next))
       return false;
-    // MD - don't bother throwing exception, since this isn't a complete check for ring validity
-//  throw new IllegalArgumentException("Invalid ring (contains repeated line segments)");
 
-    int disc = computeOrientation(prev, hip, next);
+    int disc = computeOrientation(prev, hiPt, next);
 
     /**
      *  If disc is exactly 0, lines are collinear.  There are two possible cases:
@@ -302,6 +290,7 @@ public class CGAlgorithms
       Math.abs(s) *
       Math.sqrt(((B.x - A.x) * (B.x - A.x) + (B.y - A.y) * (B.y - A.y)));
   }
+  
   /**
    * Computes the perpendicular distance from a point p
    * to the (infinite) line containing the points AB
@@ -331,6 +320,29 @@ public class CGAlgorithms
       Math.sqrt(((B.x - A.x) * (B.x - A.x) + (B.y - A.y) * (B.y - A.y)));
   }
 
+  /**
+   * Computes the distance from a point to a sequence
+   * of line segments.
+   * 
+   * @param p a point
+   * @param line a sequence of contiguous line segments defined by their vertices
+   * @return the minimum distance between the point and the line segments
+   */
+	public static double distancePointLine(Coordinate p, Coordinate[] line)
+	{
+		if (line.length == 0) 
+			throw new IllegalArgumentException("Line array must contain at least one vertex");
+		// this handles the case of length = 1
+		double minDistance = p.distance(line[0]);
+		for (int i = 0; i < line.length - 1; i++) {
+			double dist = CGAlgorithms.distancePointLine(p, line[i], line[i+1]);
+			if (dist < minDistance) {
+				minDistance = dist;
+			}
+		}
+		return minDistance;
+	}
+	
   /**
    * Computes the distance from a line segment AB to a line segment CD
    *
@@ -399,8 +411,13 @@ limiting conditions:
   }
 
   /**
-   * Returns the signed area for a ring.  The area is positive if
-   * the ring is oriented CW.
+   * Computes the signed area for a ring.      
+   * The signed area is positive if
+   * the ring is oriented CW, negative if the ring is oriented CCW,
+   * and zero if the ring is degenerate or flat. 
+   * 
+   * @param ring the coordinates forming the ring
+   * @return the signed area of the ring
    */
   public static double signedArea(Coordinate[] ring)
   {
@@ -416,19 +433,67 @@ limiting conditions:
     return -sum  / 2.0;
   }
 
+  /**
+   * Computes the signed area for a ring.      
+   * The signed area is positive if
+   * the ring is oriented CW, negative if the ring is oriented CCW,
+   * and zero if the ring is degenerate or flat. 
+   * 
+   * @param ring the coordinates forming the ring
+   * @return the signed area of the ring
+   */
+  public static double signedArea(CoordinateSequence ring)
+  {
+    int n = ring.size();
+    if (n < 3) return 0.0;
+         double sum = 0.0;
+    Coordinate p = new Coordinate();
+    ring.getCoordinate(0, p);
+    double bx = p.x;
+    double by = p.y;
+         for (int i = 1; i < n; i++) {
+      ring.getCoordinate(i, p);
+      double cx = p.x;
+      double cy = p.y;
+      sum += (bx + cx) * (cy - by);
+             bx = cx;
+      by = cy;
+    }
+    return -sum  / 2.0;
+  } 
+  
   /**
    * Computes the length of a linestring specified by a sequence of points.
    *
    * @param pts the points specifying the linestring
    * @return the length of the linestring
    */
-  public static double length(CoordinateSequence pts) {
-      if (pts.size() < 1) return 0.0;
-      double sum = 0.0;
-      for (int i = 1; i < pts.size(); i++) {
-        sum += pts.getCoordinate(i).distance(pts.getCoordinate(i - 1));
-      }
-      return sum;
+  public static double length(CoordinateSequence pts) 
+  {
+    // optimized for processing CoordinateSequences
+    int n = pts.size();
+    if (n <= 1) return 0.0;
+    
+    double len = 0.0;
+    
+    Coordinate p = new Coordinate();
+    pts.getCoordinate(0, p);
+    double x0 = p.x;
+    double y0 = p.y;
+    
+    for (int i = 1; i < n; i++) {
+      pts.getCoordinate(i, p);
+      double x1 = p.x;
+      double y1 = p.y;
+      double dx = x1 - x0;
+      double dy = y1 - y0;
+
+      len += Math.sqrt(dx * dx + dy * dy);
+      
+      x0 = x1;
+      y0 = y1;
+    }
+    return len;
   }
 
 }
