@@ -1,6 +1,3 @@
-
-
-
 /*
  * The JTS Topology Suite is a collection of Java classes that
  * implement the fundamental operations required to validate a given
@@ -37,26 +34,42 @@ package com.vividsolutions.jts.algorithm;
 
 import java.util.Iterator;
 import com.vividsolutions.jts.geom.*;
-import com.vividsolutions.jts.geomgraph.GeometryGraph;
 
 /**
- * Computes the topological relationship ({@link Location})
+ * Computes the topological ({@link Location})
  * of a single point to a {@link Geometry}.
- * The algorithm obeys the SFS Boundary Determination Rule to determine
- * whether the point lies on the boundary or not.
+ * A {@link BoundaryNodeRule} may be specified 
+ * to control the evaluation of whether the point lies on the boundary or not
+ * The default rule is to use the the <i>SFS Boundary Determination Rule</i>
  * <p>
+ * Notes:
+ * <ul>
+ * <li>{@link LinearRing}s do not enclose any area - points inside the ring are still in the EXTERIOR of the ring.
+ * </ul>
  * Instances of this class are not reentrant.
  *
  * @version 1.7
  */
 public class PointLocator
 {
+  // default is to use OGC SFS rule
+  private BoundaryNodeRule boundaryRule = 
+  	//BoundaryNodeRule.ENDPOINT_BOUNDARY_RULE; 
+  	BoundaryNodeRule.OGC_SFS_BOUNDARY_RULE;
+
   private boolean isIn;         // true if the point lies in or on any Geometry element
   private int numBoundaries;    // the number of sub-elements whose boundaries the point lies in
 
   public PointLocator() {
   }
 
+  public PointLocator(BoundaryNodeRule boundaryRule)
+  {
+    if (boundaryRule == null)
+      throw new IllegalArgumentException("Rule must be non-null");
+    this.boundaryRule = boundaryRule;
+  }
+
   /**
    * Convenience method to test a point for intersection with
    * a Geometry
@@ -83,9 +96,6 @@ public class PointLocator
   {
     if (geom.isEmpty()) return Location.EXTERIOR;
 
-    if (geom instanceof LinearRing) {
-      return locate(p, (LinearRing) geom);
-    }
     if (geom instanceof LineString) {
       return locate(p, (LineString) geom);
     }
@@ -96,15 +106,18 @@ public class PointLocator
     isIn = false;
     numBoundaries = 0;
     computeLocation(p, geom);
-    if (GeometryGraph.isInBoundary(numBoundaries)) return Location.BOUNDARY;
-    if (numBoundaries > 0 || isIn) return Location.INTERIOR;
+    if (boundaryRule.isInBoundary(numBoundaries))
+      return Location.BOUNDARY;
+    if (numBoundaries > 0 || isIn)
+      return Location.INTERIOR;
+
     return Location.EXTERIOR;
   }
 
   private void computeLocation(Coordinate p, Geometry geom)
   {
-    if (geom instanceof LinearRing) {
-      updateLocationInfo(locate(p, (LinearRing) geom));
+    if (geom instanceof Point) {
+      updateLocationInfo(locate(p, (Point) geom));
     }
     if (geom instanceof LineString) {
       updateLocationInfo(locate(p, (LineString) geom));
@@ -142,8 +155,21 @@ public class PointLocator
     if (loc == Location.BOUNDARY) numBoundaries++;
   }
 
+  private int locate(Coordinate p, Point pt)
+  {
+  	// no point in doing envelope test, since equality test is just as fast
+  	
+    Coordinate ptCoord = pt.getCoordinate();
+    if (ptCoord.equals2D(p))
+      return Location.INTERIOR;
+    return Location.EXTERIOR;
+  }
+
   private int locate(Coordinate p, LineString l)
   {
+  	// bounding-box check
+  	if (! l.getEnvelopeInternal().intersects(p)) return Location.EXTERIOR;
+  	
     Coordinate[] pt = l.getCoordinates();
     if (! l.isClosed()) {
       if (p.equals(pt[0])
@@ -156,32 +182,33 @@ public class PointLocator
     return Location.EXTERIOR;
   }
 
-  private int locate(Coordinate p, LinearRing ring)
+  private int locateInPolygonRing(Coordinate p, LinearRing ring)
   {
-    // can this test be folded into isPointInRing ?
-    if (CGAlgorithms.isOnLine(p, ring.getCoordinates())) {
-      return Location.BOUNDARY;
-    }
-    if (CGAlgorithms.isPointInRing(p, ring.getCoordinates()))
-      return Location.INTERIOR;
-    return Location.EXTERIOR;
+  	// bounding-box check
+  	if (! ring.getEnvelopeInternal().intersects(p)) return Location.EXTERIOR;
+
+  	return CGAlgorithms.locatePointInRing(p, ring.getCoordinates());
   }
 
   private int locate(Coordinate p, Polygon poly)
   {
     if (poly.isEmpty()) return Location.EXTERIOR;
+
     LinearRing shell = (LinearRing) poly.getExteriorRing();
 
-    int shellLoc = locate(p, shell);
+    int shellLoc = locateInPolygonRing(p, shell);
     if (shellLoc == Location.EXTERIOR) return Location.EXTERIOR;
     if (shellLoc == Location.BOUNDARY) return Location.BOUNDARY;
     // now test if the point lies in or on the holes
     for (int i = 0; i < poly.getNumInteriorRing(); i++) {
       LinearRing hole = (LinearRing) poly.getInteriorRingN(i);
-      int holeLoc = locate(p, hole);
+      int holeLoc = locateInPolygonRing(p, hole);
       if (holeLoc == Location.INTERIOR) return Location.EXTERIOR;
       if (holeLoc == Location.BOUNDARY) return Location.BOUNDARY;
     }
     return Location.INTERIOR;
   }
+
+
+
 }
