@@ -40,22 +40,30 @@ import com.vividsolutions.jts.util.Assert;
 /**
  * A lightweight class used to store coordinates
  * on the 2-dimensional Cartesian plane.
- *  It is distinct from <code>Point</code>, which is a subclass of <code>Geometry</code>
- *  . Unlike objects of type <code>Point</code> (which contain additional
- *  information such as an envelope, a precision model, and spatial reference
- *  system information), a <code>Coordinate</code> only contains ordinate values
- *  and accessor methods. <P>
+ * It is distinct from {@link Point}, which is a subclass of {@link Geometry}. 
+ * Unlike objects of type {@link Point} (which contain additional
+ * information such as an envelope, a precision model, and spatial reference
+ * system information), a <code>Coordinate</code> only contains ordinate values
+ * and accessor methods. <P>
  *
- *  <code>Coordinate</code>s are two-dimensional points, with an additional
- *  z-ordinate. JTS does not support any operations on the z-ordinate except
- *  the basic accessor functions. Constructed coordinates will have a
- *  z-ordinate of <code>NaN</code>.  The standard comparison functions will ignore
- *  the z-ordinate.
+ * <code>Coordinate</code>s are two-dimensional points, with an additional Z-ordinate. 
+ * JTS does not support any operations on the Z-ordinate except the basic accessor functions. 
+ * If an Z-ordinate value is not specified or not defined, 
+ * constructed coordinates have a Z-ordinate of <code>NaN</code>
+ * (which is also the value of <code>NULL_ORDINATE</code>).  
+ * The standard comparison functions ignore the Z-ordinate.
  *
  *@version 1.7
  */
 public class Coordinate implements Comparable, Cloneable, Serializable {
   private static final long serialVersionUID = 6683108902428366910L;
+  
+  /**
+   * The value used to indicate a null or missing ordinate value.
+   * In particular, used for the value of ordinates for dimensions 
+   * greater than the defined dimension of a coordinate.
+   */
+  public static final double NULL_ORDINATE = Double.NaN;
   /**
    *  The x-coordinate.
    */
@@ -106,14 +114,13 @@ public class Coordinate implements Comparable, Cloneable, Serializable {
    *@param  y  the y-value
    */
   public Coordinate(double x, double y) {
-    this(x, y, Double.NaN);
+    this(x, y, NULL_ORDINATE);
   }
 
 
 
   /**
-   *  Sets this <code>Coordinate</code>s (x,y,z) values to that of <code>other</code>
-   *  .
+   *  Sets this <code>Coordinate</code>s (x,y,z) values to that of <code>other</code>.
    *
    *@param  other  the <code>Coordinate</code> to copy
    */
@@ -224,6 +231,13 @@ public class Coordinate implements Comparable, Cloneable, Serializable {
     }
   }
 
+  /**
+   * Computes the 2-dimensional Euclidean distance to another location.
+   * The Z-ordinate is ignored.
+   * 
+   * @param p a point
+   * @return the 2-dimensional Euclidean distance between the locations
+   */
   public double distance(Coordinate p) {
     double dx = x - p.x;
     double dy = y - p.y;
@@ -231,6 +245,11 @@ public class Coordinate implements Comparable, Cloneable, Serializable {
     return Math.sqrt(dx * dx + dy * dy);
   }
 
+  /**
+   * Gets a hashcode for this coordinate.
+   * 
+   * @return a hashcode for this coordinate
+   */
   public int hashCode() {
     //Algorithm from Effective Java by Joshua Bloch [Jon Aquino]
     int result = 17;
@@ -240,8 +259,10 @@ public class Coordinate implements Comparable, Cloneable, Serializable {
   }
 
   /**
-   * Returns a hash code for a double value, using the algorithm from
+   * Computes a hash code for a double value, using the algorithm from
    * Joshua Bloch's book <i>Effective Java"</i>
+   * 
+   * @return a hashcode for the double value
    */
   public static int hashCode(double x) {
     long f = Double.doubleToLongBits(x);
