@@ -33,19 +33,23 @@
 package com.vividsolutions.jts.geom;
 
 import java.io.Serializable;
-import java.util.Collection;
-import java.util.Iterator;
+import java.util.*;
 
 import com.vividsolutions.jts.algorithm.*;
+import com.vividsolutions.jts.geom.util.*;
 import com.vividsolutions.jts.io.WKTWriter;
+import com.vividsolutions.jts.operation.*;
 import com.vividsolutions.jts.operation.buffer.BufferOp;
 import com.vividsolutions.jts.operation.distance.DistanceOp;
 import com.vividsolutions.jts.operation.overlay.OverlayOp;
+import com.vividsolutions.jts.operation.union.UnaryUnionOp;
+import com.vividsolutions.jts.operation.overlay.snap.SnapIfNeededOverlayOp;
 import com.vividsolutions.jts.operation.predicate.RectangleIntersects;
 import com.vividsolutions.jts.operation.predicate.RectangleContains;
 import com.vividsolutions.jts.operation.relate.RelateOp;
 import com.vividsolutions.jts.operation.valid.IsValidOp;
 import com.vividsolutions.jts.util.Assert;
+
 /**
  * The base class for all geometric objects.
  * <P>
@@ -128,45 +132,47 @@ import com.vividsolutions.jts.util.Assert;
 public abstract class Geometry
     implements Cloneable, Comparable, Serializable
 {
-    private static final long serialVersionUID = 8763622679187376702L;
-
-  /**
-   *  The bounding box of this <code>Geometry</code>.
-   */
-  protected Envelope envelope;
-
-  private final static Class[] sortedClasses = new Class[] {
-      Point.class,
-      MultiPoint.class,
-      LineString.class,
-      LinearRing.class,
-      MultiLineString.class,
-      Polygon.class,
-      MultiPolygon.class,
-      GeometryCollection.class
-      };
-
+  private static final long serialVersionUID = 8763622679187376702L;
+    
+  private static Class[] sortedClasses;  
+  
   private final static GeometryComponentFilter geometryChangedFilter = new GeometryComponentFilter() {
     public void filter(Geometry geom) {
       geom.geometryChangedAction();
     }
   };
 
-  public Geometry(GeometryFactory factory) {
-    this.factory = factory;
-    this.SRID = factory.getSRID();
-  }
+  /**
+   *  The bounding box of this <code>Geometry</code>.
+   */
+  protected Envelope envelope;
 
-  private GeometryFactory factory;
-
-  // MD - no longer used.  Remove in next version
-  //private static final GeometryFactory INTERNAL_GEOMETRY_FACTORY = new GeometryFactory();
+  /**
+   * The {@link GeometryFactory} used to create this Geometry
+   */
+  protected final GeometryFactory factory;
 
   /**
    *  The ID of the Spatial Reference System used by this <code>Geometry</code>
    */
   protected int SRID;
 
+  /**
+   * An object reference which can be used to carry ancillary data defined
+   * by the client.
+   */
+  private Object userData = null;
+
+  /**
+   * Creates a new <tt>Geometry</tt> via the specified GeometryFactory.
+   *
+   * @param factory
+   */
+  public Geometry(GeometryFactory factory) {
+    this.factory = factory;
+    this.SRID = factory.getSRID();
+  }
+
   /**
    *  Returns the name of this object's <code>com.vivid.jts.geom</code>
    *  interface.
@@ -174,9 +180,6 @@ public abstract class Geometry
    *@return    the name of this <code>Geometry</code>s most specific <code>com.vividsolutions.jts.geom</code>
    *      interface
    */
-  //I wonder if we need this method, now that we have renamed the classes to
-  //what their old interfaces were named. Now we can perhaps simply use
-  //getClass().getName(). Who calls this method anyway? [Jon Aquino]
   public abstract String getGeometryType();
 
   /**
@@ -212,8 +215,6 @@ public abstract class Geometry
     return false;
   }
 
-
-
   /**
    *  Returns the ID of the Spatial Reference System used by the <code>Geometry</code>.
    *  <P>
@@ -233,13 +234,19 @@ public abstract class Geometry
   }
     /**
    *  Sets the ID of the Spatial Reference System used by the <code>Geometry</code>.
+   *  <p>
+   *  <b>NOTE:</b> This method should only be used for exceptional circumstances or 
+   *  for backwards compatibility.  Normally the SRID should be set on the 
+   *  {@link GeometryFactory} used to create the geometry.
+   *  SRIDs set using this method will <i>not</i> be propagated to 
+   *  geometries returned by constructive methods.
+   *  
+   *  @see GeometryFactory
    */
   public void setSRID(int SRID) {
     this.SRID = SRID;
   }
 
-  private Object userData = null;
-
   /**
    * Gets the factory which contains the context in which this geometry was created.
    *
@@ -306,19 +313,34 @@ public abstract class Geometry
   }
 
   /**
-   *  Returns a vertex of this <code>Geometry</code>.
+   *  Returns a vertex of this <code>Geometry</code>
+   *  (usually, but not necessarily, the first one).
+   *  The returned coordinate should not be assumed
+   *  to be an actual Coordinate object used in
+   *  the internal representation.
    *
    *@return    a {@link Coordinate} which is a vertex of this <code>Geometry</code>.
-   *          Returns <code>null</code> if this Geometry is empty
+   *@return null if this Geometry is empty
    */
   public abstract Coordinate getCoordinate();
+  
   /**
-   *  Returns this <code>Geometry</code> s vertices. If you modify the coordinates
-   *  in this array, be sure to call #geometryChanged afterwards.
-   *  The <code>Geometry</code>s contained by composite <code>Geometry</code>s
-   *  must be Geometry's; that is, they must implement <code>getCoordinates</code>.
+   *  Returns an array containing the values of all the vertices for 
+   *  this geometry.
+   *  If the geometry is a composite, the array will contain all the vertices
+   *  for the components, in the order in which the components occur in the geometry.
+   *  <p>
+   *  In general, the array cannot be assumed to be the actual internal 
+   *  storage for the vertices.  Thus modifying the array
+   *  may not modify the geometry itself. 
+   *  Use the {@link CoordinateSequence#setOrdinate} method
+   *  (possibly on the components) to modify the underlying data.
+   *  If the coordinates are modified, 
+   *  {@link #geometryChanged} must be called afterwards.
    *
    *@return    the vertices of this <code>Geometry</code>
+   *@see geometryChanged
+   *@see CoordinateSequence#setOrdinate
    */
   public abstract Coordinate[] getCoordinates();
 
@@ -354,7 +376,12 @@ public abstract class Geometry
    *      self-tangency, self-intersection or other anomalous points
    * @see #isValid
    */
-  public abstract boolean isSimple();
+  public boolean isSimple()
+  {
+    checkNotGeometryCollection(this);
+    IsSimpleOp op = new IsSimpleOp(this);
+    return op.isSimple();
+  }
 
   /**
    *  Tests the validity of this <code>Geometry</code>.
@@ -383,7 +410,10 @@ public abstract class Geometry
    *  Returns the minimum distance between this <code>Geometry</code>
    *  and the <code>Geometry</code> g
    *
-   *@param  g  the <code>Geometry</code> from which to compute the distance
+   * @param  g  the <code>Geometry</code> from which to compute the distance
+   * @return the distance between the geometries
+   * @return 0 if either input geometry is empty
+   * @throws IllegalArgumentException if g is null
    */
   public double distance(Geometry g)
   {
@@ -403,11 +433,13 @@ public abstract class Geometry
     double envDist = getEnvelopeInternal().distance(geom.getEnvelopeInternal());
     if (envDist > distance)
       return false;
-    // NOTE: this could be implemented more efficiently
+    return DistanceOp.isWithinDistance(this, geom, distance);
+    /*
     double geomDist = this.distance(geom);
     if (geomDist > distance)
       return false;
     return true;
+    */
   }
 
   public boolean isRectangle()
@@ -504,17 +536,29 @@ public abstract class Geometry
   }
 
   /**
-   *  Returns the dimension of this <code>Geometry</code>.
+   * Returns the dimension of this geometry.
+   * The dimension of a geometry is is the topological 
+   * dimension of its embedding in the 2-D Euclidean plane.
+   * In the JTS spatial model, dimension values are in the set {0,1,2}.
+   * <p>
+   * Note that this is a different concept to the dimension of 
+   * the vertex {@link Coordinate}s.  
+   * The geometry dimension can never be greater than the coordinate dimension.
+   * For example, a 0-dimensional geometry (e.g. a Point) 
+   * may have a coordinate dimension of 3 (X,Y,Z). 
    *
-   *@return    the dimension of the class implementing this interface, whether
-   *      or not this object is the empty geometry
+   *@return the topological dimension of this geometry.
    */
   public abstract int getDimension();
+
   /**
-   *  Returns the boundary, or the empty geometry if this <code>Geometry</code>
-   *  is empty. For a discussion of this function, see the OpenGIS Simple
-   *  Features Specification. As stated in SFS Section 2.1.13.1, "the boundary
-   *  of a Geometry is a set of Geometries of the next lower dimension."
+   * Returns the boundary, or an empty geometry of appropriate dimension
+   * if this <code>Geometry</code>  is empty.
+   * (In the case of zero-dimensional geometries, '
+   * an empty GeometryCollection is returned.)
+   * For a discussion of this function, see the OpenGIS Simple
+   * Features Specification. As stated in SFS Section 2.1.13.1, "the boundary
+   * of a Geometry is a set of Geometries of the next lower dimension."
    *
    *@return    the closure of the combinatorial boundary of this <code>Geometry</code>
    */
@@ -559,9 +603,10 @@ public abstract class Geometry
   }
 
   /**
-   * Notifies this Geometry that its Coordinates have been changed by an external
-   * party (using a CoordinateFilter, for example). The Geometry will flush
-   * and/or update any information it has cached (such as its {@link Envelope} ).
+   * Notifies this geometry that its coordinates have been changed by an external
+   * party (for example, via a {@link CoordinateFilter}). 
+   * When this method is called the geometry will flush
+   * and/or update any derived information it has cached (such as its {@link Envelope} ).
    */
   public void geometryChanged() {
     apply(geometryChangedFilter);
@@ -578,13 +623,14 @@ public abstract class Geometry
   }
 
   /**
-   * Returns <code>true</code> if this geometry is disjoint to the specified geometry.
+   * Tests whether this geometry is disjoint from the specified geometry.
    * <p>
    * The <code>disjoint</code> predicate has the following equivalent definitions:
    * <ul>
    * <li>The two geometries have no point in common
-   * <li>The DE-9IM Intersection Matrix for the two geometries is FF*FF****
-   * <li>! <code>g.intersects(this)</code>
+   * <li>The DE-9IM Intersection Matrix for the two geometries matches 
+   * <code>[FF*FF****]</code>
+   * <li><code>! g.intersects(this)</code>
    * (<code>disjoint</code> is the inverse of <code>intersects</code>)
    * </ul>
    *
@@ -599,14 +645,14 @@ public abstract class Geometry
   }
 
   /**
-   * Returns <code>true</code> if this geometry touches the
+   * Tests whether this geometry touches the
    * specified geometry.
    * <p>
    * The <code>touches</code> predicate has the following equivalent definitions:
    * <ul>
    * <li>The geometries have at least one point in common, but their interiors do not intersect.
-   * <li>The DE-9IM Intersection Matrix for the two geometries is
-   *   FT*******, F**T***** or F***T****
+   * <li>The DE-9IM Intersection Matrix for the two geometries matches
+   *   <code>[FT*******]</code> or <code>[F**T*****]</code> or <code>[F***T****]</code>
    * </ul>
    * If both geometries have dimension 0, this predicate returns <code>false</code>
    *
@@ -622,12 +668,17 @@ public abstract class Geometry
   }
 
   /**
-   * Returns <code>true</code> if this geometry intersects the specified geometry.
+   * Tests whether this geometry intersects the specified geometry.
    * <p>
    * The <code>intersects</code> predicate has the following equivalent definitions:
    * <ul>
    * <li>The two geometries have at least one point in common
-   * <li>! <code>g.disjoint(this)</code>
+   * <li>The DE-9IM Intersection Matrix for the two geometries matches
+   *    <code>[T********]</code>
+   * or <code>[*T*******]</code>
+   * or <code>[***T*****]</code>
+   * or <code>[****T****]</code>
+   * <li><code>! g.disjoint(this)</code>
    * (<code>intersects</code> is the inverse of <code>disjoint</code>)
    * </ul>
    *
@@ -670,24 +721,24 @@ public abstract class Geometry
   }
 
   /**
-   * Returns <code>true</code> if this geometry crosses the
+   * Tests whether this geometry crosses the
    * specified geometry.
    * <p>
    * The <code>crosses</code> predicate has the following equivalent definitions:
    * <ul>
    * <li>The geometries have some but not all interior points in common.
-   * <li>The DE-9IM Intersection Matrix for the two geometries is
+   * <li>The DE-9IM Intersection Matrix for the two geometries matches
    *   <ul>
-   *    <li>T*T****** (for P/L, P/A, and L/A situations)
-   *    <li>T*****T** (for L/P, L/A, and A/L situations)
-   *    <li>0******** (for L/L situations)
+   *    <li><code>[T*T******]</code> (for P/L, P/A, and L/A situations)
+   *    <li><code>[T*****T**]</code> (for L/P, A/P, and A/L situations)
+   *    <li><code>[0********]</code> (for L/L situations)
    *   </ul>
    * </ul>
    * For any other combination of dimensions this predicate returns <code>false</code>.
    * <p>
    * The SFS defined this predicate only for P/L, P/A, L/L, and L/A situations.
-   * JTS extends the definition to apply to L/P, A/P and A/L situations as well.
-   * This makes the relation symmetric.
+   * JTS extends the definition to apply to L/P, A/P and A/L situations as well,
+   * in order to make the relation symmetric.
    *
    *@param  g  the <code>Geometry</code> with which to compare this <code>Geometry</code>
    *@return        <code>true</code> if the two <code>Geometry</code>s cross.
@@ -700,21 +751,22 @@ public abstract class Geometry
   }
 
   /**
-   * Returns <code>true</code> if this geometry is within the
+   * Tests whether this geometry is within the
    * specified geometry.
    * <p>
    * The <code>within</code> predicate has the following equivalent definitions:
    * <ul>
    * <li>Every point of this geometry is a point of the other geometry,
    * and the interiors of the two geometries have at least one point in common.
-   * <li>The DE-9IM Intersection Matrix for the two geometries is T*F**F***
+   * <li>The DE-9IM Intersection Matrix for the two geometries matches 
+   * <code>[T*F**F***]</code>
    * <li><code>g.contains(this)</code>
-   * (<code>within</code> is the inverse of <code>contains</code>)
+   * (<code>within</code> is the converse of <code>contains</code>)
    * </ul>
    * An implication of the definition is that
-   * "The boundary of a Polygon is not within the Polygon".
-   * In other words, if a geometry G is a subset of
-   * the points in the boundary of a polygon P, <code>G.within(P) = false</code>
+   * "The boundary of a Geometry is not within the Geometry".
+   * In other words, if a geometry A is a subset of
+   * the points in the boundary of a geomtry B, <code>A.within(B) = false</code>
    *
    *@param  g  the <code>Geometry</code> with which to compare this <code>Geometry</code>
    *@return        <code>true</code> if this <code>Geometry</code> is within
@@ -727,20 +779,21 @@ public abstract class Geometry
   }
 
   /**
-   * Returns <code>true</code> if this geometry contains the
+   * Tests whether this geometry contains the
    * specified geometry.
    * <p>
    * The <code>contains</code> predicate has the following equivalent definitions:
    * <ul>
    * <li>Every point of the other geometry is a point of this geometry,
    * and the interiors of the two geometries have at least one point in common.
-   * <li>The DE-9IM Intersection Matrix for the two geometries is <code>T*****FF*</code>
+   * <li>The DE-9IM Intersection Matrix for the two geometries matches 
+   * <code>[T*****FF*]</code>
    * <li><code>g.within(this)</code>
-   * (<code>contains</code> is the inverse of <code>within</code>)
+   * (<code>contains</code> is the converse of <code>within</code>)
    * </ul>
-   * An implication of the definition is that "Polygons do not
-   * contain their boundary".  In other words, if a geometry G is a subset of
-   * the points in the boundary of a polygon P, <code>P.contains(G) = false</code>
+   * An implication of the definition is that "Geometries do not
+   * contain their boundary".  In other words, if a geometry A is a subset of
+   * the points in the boundary of a geometry B, <code>B.contains(A) = false</code>
    *
    *@param  g  the <code>Geometry</code> with which to compare this <code>Geometry</code>
    *@return        <code>true</code> if this <code>Geometry</code> contains <code>g</code>
@@ -760,18 +813,19 @@ public abstract class Geometry
   }
 
   /**
-   * Returns <code>true</code> if this geometry overlaps the
+   * Tests whether this geometry overlaps the
    * specified geometry.
    * <p>
    * The <code>overlaps</code> predicate has the following equivalent definitions:
    * <ul>
-   * <li>The geometries have some but not all points in common,
+   * <li>The geometries have at least one point each not shared by the other
+   * (or equivalently neither covers the other),
    * they have the same dimension,
    * and the intersection of the interiors of the two geometries has
    * the same dimension as the geometries themselves.
-   * <li>The DE-9IM Intersection Matrix for the two geometries is
-   *   <code>T*T***T**</code> (for two points or two surfaces)
-   *   or <code>1*T***T**</code> (for two curves)
+   * <li>The DE-9IM Intersection Matrix for the two geometries matches
+   *   <code>[T*T***T**]</code> (for two points or two surfaces)
+   *   or <code>[1*T***T**]</code> (for two curves)
    * </ul>
    * If the geometries are of different dimension this predicate returns <code>false</code>.
    *
@@ -786,22 +840,24 @@ public abstract class Geometry
   }
 
   /**
-   * Returns <code>true</code> if this geometry covers the
+   * Tests whether this geometry covers the
    * specified geometry.
    * <p>
    * The <code>covers</code> predicate has the following equivalent definitions:
    * <ul>
    * <li>Every point of the other geometry is a point of this geometry.
-   * <li>The DE-9IM Intersection Matrix for the two geometries is
-   *    <code>T*****FF*</code>
-   * or <code>*T****FF*</code>
-   * or <code>***T**FF*</code>
-   * or <code>****T*FF*</code>
+   * <li>The DE-9IM Intersection Matrix for the two geometries matches
+   *    <code>[T*****FF*]</code>
+   * or <code>[*T****FF*]</code>
+   * or <code>[***T**FF*]</code>
+   * or <code>[****T*FF*]</code>
    * <li><code>g.coveredBy(this)</code>
-   * (<code>covers</code> is the inverse of <code>coverdBy</code>)
+   * (<code>covers</code> is the converse of <code>coveredBy</code>)
    * </ul>
-   * Note the difference between <code>covers</code> and <code>contains</code>
-   * - <code>covers</code> is a more inclusive relation.
+   * If either geometry is empty, the value of this predicate is <tt>false</tt>.
+   * <p>
+   * This predicate is similar to {@link #contains},
+   * but is more inclusive (i.e. returns <tt>true</tt> for more cases).
    * In particular, unlike <code>contains</code> it does not distinguish between
    * points in the boundary and in the interior of geometries.
    * For most situations, <code>covers</code> should be used in preference to <code>contains</code>.
@@ -816,32 +872,35 @@ public abstract class Geometry
    */
   public boolean covers(Geometry g) {
     // short-circuit test
-    if (! getEnvelopeInternal().contains(g.getEnvelopeInternal()))
+    if (! getEnvelopeInternal().covers(g.getEnvelopeInternal()))
       return false;
     // optimization for rectangle arguments
     if (isRectangle()) {
-      return getEnvelopeInternal().contains(g.getEnvelopeInternal());
+    	// since we have already tested that the test envelope is covered
+      return true;
     }
     return relate(g).isCovers();
   }
 
   /**
-   * Returns <code>true</code> if this geometry is covered by the
+   * Tests whether this geometry is covered by the
    * specified geometry.
    * <p>
    * The <code>coveredBy</code> predicate has the following equivalent definitions:
    * <ul>
    * <li>Every point of this geometry is a point of the other geometry.
-   * <li>The DE-9IM Intersection Matrix for the two geometries is
-   *    <code>T*F**F***</code>
-   * or <code>*TF**F***</code>
-   * or <code>**FT*F***</code>
-   * or <code>**F*TF***</code>
+   * <li>The DE-9IM Intersection Matrix for the two geometries matches
+   *    <code>[T*F**F***]</code>
+   * or <code>[*TF**F***]</code>
+   * or <code>[**FT*F***]</code>
+   * or <code>[**F*TF***]</code>
    * <li><code>g.covers(this)</code>
-   * (<code>coveredBy</code> is the inverse of <code>covers</code>)
+   * (<code>coveredBy</code> is the converse of <code>covers</code>)
    * </ul>
-   * Note the difference between <code>coveredBy</code> and <code>within</code>
-   * - <code>coveredBy</code> is a more inclusive relation
+   * If either geometry is empty, the value of this predicate is <tt>false</tt>.
+   * <p>
+   * This predicate is similar to {@link #within},
+   * but is more inclusive (i.e. returns <tt>true</tt> for more cases).
    *
    *@param  g  the <code>Geometry</code> with which to compare this <code>Geometry</code>
    *@return        <code>true</code> if this <code>Geometry</code> is covered by <code>g</code>
@@ -854,7 +913,7 @@ public abstract class Geometry
   }
 
   /**
-   *  Returns <code>true</code> if the elements in the DE-9IM
+   * Tests whether the elements in the DE-9IM
    * {@link IntersectionMatrix} for the two <code>Geometry</code>s match the elements in <code>intersectionPattern</code>.
    * The pattern is a 9-character string, with symbols drawn from the following set:
    *  <UL>
@@ -894,7 +953,7 @@ public abstract class Geometry
   }
 
   /**
-   * Returns <code>true</code> if this geometry is equal to the
+   * Tests whether this geometry is equal to the
    * specified geometry.
    * <p>
    * The <code>equals</code> predicate has the following equivalent definitions:
@@ -903,6 +962,8 @@ public abstract class Geometry
    * and no point of either geometry lies in the exterior of the other geometry.
    * <li>The DE-9IM Intersection Matrix for the two geometries is T*F**FFF*
    * </ul>
+   * <b>Note</b> that this method computes topologically equality, not structural or
+   * vertex-wise equality.
    *
    *@param  other  the <code>Geometry</code> with which to compare this <code>Geometry</code>
    *@return        <code>true</code> if the two <code>Geometry</code>s are equal
@@ -914,8 +975,6 @@ public abstract class Geometry
     return relate(g).isEquals(getDimension(), g.getDimension());
   }
 
-  //<<PERHAPS:DESIGN>> Override Object#equals [Jon Aquino]
-
   public String toString() {
     return toText();
   }
@@ -933,40 +992,64 @@ public abstract class Geometry
   }
 
   /**
-   * Computes a buffer area around this geometry having the given
-   * width.
-   * The buffer of a Geometry is the Minkowski sum or difference
-   * of the geometry with a disc of radius <code>abs(distance)</code>.
-   * The buffer is constructed using 8 segments per quadrant to represent curves.
-   * The end cap style is <tt>CAP_ROUND</tt>.
-   *
-   *@param  distance  the width of the buffer (may be positive, negative or 0)
-   *@return an area geometry representing the buffer region
-   *
-   * @see #buffer(double, int)
-   * @see #buffer(double, int, int)
-   */
-  public Geometry buffer(double distance) {
-    return BufferOp.bufferOp(this, distance);
-  }
+	 * Computes a buffer area around this geometry having the given width. The
+	 * buffer of a Geometry is the Minkowski sum or difference of the geometry
+	 * with a disc of radius <code>abs(distance)</code>.
+	 * <p> 
+	 * Mathematically-exact buffer area boundaries can contain circular arcs. 
+	 * To represent these arcs using linear geometry they must be approximated with line segments.
+	 * The buffer geometry is constructed using 8 segments per quadrant to approximate 
+	 * the circular arcs.
+	 * The end cap style is <tt>CAP_ROUND</tt>.
+	 * <p>
+	 * The buffer operation always returns a polygonal result. The negative or
+	 * zero-distance buffer of lines and points is always an empty {@link Polygon}.
+	 * This is also the result for the buffers of degenerate (zero-area) polygons.
+	 * 
+	 * @param distance
+	 *          the width of the buffer (may be positive, negative or 0)
+	 * @return a polygonal geometry representing the buffer region (which may be
+	 *         empty)
+	 * 
+	 * @throws TopologyException
+	 *           if a robustness error occurs
+	 * 
+	 * @see #buffer(double, int)
+	 * @see #buffer(double, int, int)
+	 */
+	public Geometry buffer(double distance) {
+		return BufferOp.bufferOp(this, distance);
+	}
 
   /**
-   * Computes a buffer area around this geometry having the given
-   * width and with a specified accuracy of approximation for circular arcs.
-   * <p>
-   * Buffer area boundaries can contain circular arcs.
-   * To represent these arcs using linear geometry they must be approximated with line segments.
-   * The <code>quadrantSegments</code> argument allows controlling the
-   * accuracy of the approximation
-   * by specifying the number of line segments used to represent a quadrant of a circle
-   *
-   *@param  distance  the width of the buffer (may be positive, negative or 0)
-   *@param quadrantSegments the number of line segments used to represent a quadrant of a circle
-   *@return an area geometry representing the buffer region
-   *
-   * @see #buffer(double)
-   * @see #buffer(double, int, int)
-   */
+	 * Computes a buffer area around this geometry having the given width and with
+	 * a specified accuracy of approximation for circular arcs.
+	 * <p>
+	 * Mathematically-exact buffer area boundaries can contain circular arcs. 
+	 * To represent these arcs
+	 * using linear geometry they must be approximated with line segments. The
+	 * <code>quadrantSegments</code> argument allows controlling the accuracy of
+	 * the approximation by specifying the number of line segments used to
+	 * represent a quadrant of a circle
+	 * <p>
+	 * The buffer operation always returns a polygonal result. The negative or
+	 * zero-distance buffer of lines and points is always an empty {@link Polygon}.
+	 * This is also the result for the buffers of degenerate (zero-area) polygons.
+	 * 
+	 * @param distance
+	 *          the width of the buffer (may be positive, negative or 0)
+	 * @param quadrantSegments
+	 *          the number of line segments used to represent a quadrant of a
+	 *          circle
+	 * @return a polygonal geometry representing the buffer region (which may be
+	 *         empty)
+	 * 
+	 * @throws TopologyException
+	 *           if a robustness error occurs
+	 * 
+	 * @see #buffer(double)
+	 * @see #buffer(double, int, int)
+	 */
   public Geometry buffer(double distance, int quadrantSegments) {
     return BufferOp.bufferOp(this, distance, quadrantSegments);
   }
@@ -976,7 +1059,7 @@ public abstract class Geometry
    * width and with a specified accuracy of approximation for circular arcs,
    * and using a specified end cap style.
    * <p>
-   * Buffer area boundaries can contain circular arcs.
+   * Mathematically-exact buffer area boundaries can contain circular arcs.
    * To represent these arcs using linear geometry they must be approximated with line segments.
    * The <code>quadrantSegments</code> argument allows controlling the
    * accuracy of the approximation
@@ -989,11 +1072,17 @@ public abstract class Geometry
    * <li><tt>BufferOp.CAP_BUTT</tt> - a straight line perpendicular to the end segment
    * <li><tt>BufferOp.CAP_SQUARE</tt> - a half-square
    * </ul>
+	 * <p>
+	 * The buffer operation always returns a polygonal result. The negative or
+	 * zero-distance buffer of lines and points is always an empty {@link Polygon}.
+	 * This is also the result for the buffers of degenerate (zero-area) polygons.
    *
    *@param  distance  the width of the buffer (may be positive, negative or 0)
    *@param quadrantSegments the number of line segments used to represent a quadrant of a circle
    *@param endCapStyle the end cap style to use
-   *@return an area geometry representing the buffer region
+   *@return a polygonal geometry representing the buffer region (which may be empty)
+   *
+   * @throws TopologyException if a robustness error occurs
    *
    * @see #buffer(double)
    * @see #buffer(double, int)
@@ -1038,18 +1127,52 @@ public abstract class Geometry
   }
 
   /**
-   *  Computes a <code>Geometry</code> representing the points shared by this
-   *  <code>Geometry</code> and <code>other</code>.
-   *
-   * @param  other  the <code>Geometry</code> with which to compute the
-   *      intersection
-   * @return        the points common to the two <code>Geometry</code>s
-   * @throws TopologyException if a robustness error occurs
+   * Computes a new geometry which has all component coordinate sequences
+   * in reverse order (opposite orientation) to this one.
+   * 
+   * @return a reversed geometry
    */
-  public Geometry intersection(Geometry other) {
+  public abstract Geometry reverse();
+  
+  /**
+   * Computes a <code>Geometry</code> representing the points shared by this
+   * <code>Geometry</code> and <code>other</code>.
+   * {@link GeometryCollection}s support intersection with 
+   * homogeneous collection types, with the semantics that
+   * the result is a {@link GeometryCollection} of the
+   * intersection of each element of the target with the argument. 
+   *
+   * @param  other the <code>Geometry</code> with which to compute the intersection
+   * @return the points common to the two <code>Geometry</code>s
+   * @throws TopologyException if a robustness error occurs
+   * @throws IllegalArgumentException if the argument is a non-empty GeometryCollection
+   */
+  public Geometry intersection(Geometry other)
+  {
+  	/**
+  	 * TODO: MD - add optimization for P-A case using Point-In-Polygon
+  	 */
+    // special case: if one input is empty ==> empty
+    if (this.isEmpty()) return this.getFactory().createGeometryCollection(null);
+    if (other.isEmpty()) return this.getFactory().createGeometryCollection(null);
+
+    // compute for GCs
+    if (isGeometryCollection(this)) {
+      final Geometry g2 = other;
+      return GeometryCollectionMapper.map(
+          (GeometryCollection) this,
+          new GeometryCollectionMapper.MapOp() {
+        public Geometry map(Geometry g) {
+          return g.intersection(g2);
+        }
+      });
+    }
+//    if (isGeometryCollection(other))
+//      return other.intersection(this);
+    
     checkNotGeometryCollection(this);
     checkNotGeometryCollection(other);
-    return OverlayOp.overlayOp(this, other, OverlayOp.INTERSECTION);
+    return SnapIfNeededOverlayOp.overlayOp(this, other, OverlayOp.INTERSECTION);
   }
 
   /**
@@ -1060,11 +1183,19 @@ public abstract class Geometry
    *@return        a set combining the points of this <code>Geometry</code> and
    *      the points of <code>other</code>
    * @throws TopologyException if a robustness error occurs
+   * @throws IllegalArgumentException if either input is a non-empty GeometryCollection
    */
-  public Geometry union(Geometry other) {
+  public Geometry union(Geometry other)
+  {
+    // special case: if either input is empty ==> other input
+    if (this.isEmpty()) return (Geometry) other.clone();
+    if (other.isEmpty()) return (Geometry) clone();
+
+    // TODO: optimize if envelopes of geometries do not intersect
+    
     checkNotGeometryCollection(this);
     checkNotGeometryCollection(other);
-    return OverlayOp.overlayOp(this, other, OverlayOp.UNION);
+    return SnapIfNeededOverlayOp.overlayOp(this, other, OverlayOp.UNION);
   }
 
   /**
@@ -1077,11 +1208,20 @@ public abstract class Geometry
    *@return        the point set difference of this <code>Geometry</code> with
    *      <code>other</code>
    * @throws TopologyException if a robustness error occurs
+   * @throws IllegalArgumentException if either input is a non-empty GeometryCollection
    */
-  public Geometry difference(Geometry other) {
+  public Geometry difference(Geometry other)
+  {
+    // mod to handle empty cases better - return type of input
+    //if (this.isEmpty() || other.isEmpty()) return (Geometry) clone();
+    
+    // special case: if A.isEmpty ==> empty; if B.isEmpty ==> A
+    if (this.isEmpty()) return this.getFactory().createGeometryCollection(null);
+    if (other.isEmpty()) return (Geometry) clone();
+
     checkNotGeometryCollection(this);
     checkNotGeometryCollection(other);
-    return OverlayOp.overlayOp(this, other, OverlayOp.DIFFERENCE);
+    return SnapIfNeededOverlayOp.overlayOp(this, other, OverlayOp.DIFFERENCE);
   }
 
   /**
@@ -1095,13 +1235,40 @@ public abstract class Geometry
    *@return        the point set symmetric difference of this <code>Geometry</code>
    *      with <code>other</code>
    * @throws TopologyException if a robustness error occurs
+   * @throws IllegalArgumentException if either input is a non-empty GeometryCollection
    */
-  public Geometry symDifference(Geometry other) {
+  public Geometry symDifference(Geometry other)
+  {
+    // special case: if either input is empty ==> other input
+    if (this.isEmpty()) return (Geometry) other.clone();
+    if (other.isEmpty()) return (Geometry) clone();
+
     checkNotGeometryCollection(this);
     checkNotGeometryCollection(other);
-    return OverlayOp.overlayOp(this, other, OverlayOp.SYMDIFFERENCE);
+    return SnapIfNeededOverlayOp.overlayOp(this, other, OverlayOp.SYMDIFFERENCE);
   }
 
+	/**
+	 * Computes the union of all the elements of this geometry. Heterogeneous
+	 * {@link GeometryCollection}s are fully supported.
+	 * <p>
+	 * The result obeys the following contract:
+	 * <ul>
+	 * <li>Unioning a set of {@link LineString}s has the effect of fully noding
+	 * and dissolving the linework.
+	 * <li>Unioning a set of {@link Polygon}s will always 
+	 * return a {@link Polygonal} geometry (unlike {link #union(Geometry)},
+	 * which may return geometrys of lower dimension if a topology collapse occurred.
+	 * </ul>
+	 * 
+	 * @return
+	 * 
+	 * @see UnaryUnionOp
+	 */
+	public Geometry union() {
+		return UnaryUnionOp.union(this);
+	}
+  
   /**
    * Returns true if the two <code>Geometry</code>s are exactly equal,
    * up to a specified distance tolerance.
@@ -1147,18 +1314,29 @@ public abstract class Geometry
 
   /**
    *  Performs an operation with or on this <code>Geometry</code>'s
-   *  coordinates. If you are using this method to modify the geometry, be sure
-   *  to call #geometryChanged() afterwards. Note that you cannot use this
-   *  method to
+   *  coordinates. 
+   *  If this method modifies any coordinate values,
+   *  {@link #geometryChanged} must be called to update the geometry state. 
+   *  Note that you cannot use this method to
    *  modify this Geometry if its underlying CoordinateSequence's #get method
    *  returns a copy of the Coordinate, rather than the actual Coordinate stored
-   *  (if it even stores Coordinates at all).
+   *  (if it even stores Coordinate objects at all).
    *
    *@param  filter  the filter to apply to this <code>Geometry</code>'s
    *      coordinates
    */
   public abstract void apply(CoordinateFilter filter);
 
+  /**
+   *  Performs an operation on the coordinates in this <code>Geometry</code>'s
+   *  {@link CoordinateSequence}s. 
+   *  If the filter reports that a coordinate value has been changed, 
+   *  {@link #geometryChanged} will be called automatically.
+   *
+   *@param  filter  the filter to apply
+   */
+  public abstract void apply(CoordinateSequenceFilter filter);
+
   /**
    *  Performs an operation with or on this <code>Geometry</code> and its
    *  subelement <code>Geometry</code>s (if any).
@@ -1180,6 +1358,14 @@ public abstract class Geometry
    */
   public abstract void apply(GeometryComponentFilter filter);
 
+  /**
+   * Creates and returns a full copy of this {@link Geometry} object
+   * (including all coordinates contained by it).
+   * Subclasses are responsible for overriding this method and copying
+   * their internal data.  Overrides should call this method first.
+   *
+   * @return a clone of this instance
+   */
   public Object clone() {
     try {
       Geometry clone = (Geometry) super.clone();
@@ -1326,6 +1512,10 @@ public abstract class Geometry
     }
   }
 
+  protected boolean isGeometryCollection(Geometry g)
+  {
+    return g.getClass().equals(com.vividsolutions.jts.geom.GeometryCollection.class);
+  }
 
   /**
    *  Returns the minimum and maximum x and y values in this <code>Geometry</code>
@@ -1404,15 +1594,30 @@ public abstract class Geometry
   }
 
   private int getClassSortIndex() {
-    for (int i = 0; i < sortedClasses.length; i++) {
-      if (sortedClasses[i].isInstance(this)) {
-        return i;
-      }
-    }
-    Assert.shouldNeverReachHere("Class not supported: " + this.getClass());
-    return -1;
-  }
+		if (sortedClasses == null)
+			initSortedClasses();
 
+		for (int i = 0; i < sortedClasses.length; i++) {
+			if (sortedClasses[i].isInstance(this))
+				return i;
+		}
+		Assert.shouldNeverReachHere("Class not supported: " + this.getClass());
+		return -1;
+	}
+
+  private static void initSortedClasses()
+  {
+		sortedClasses = new Class[] { 
+					Point.class, 
+					MultiPoint.class,
+					LineString.class, 
+					LinearRing.class, 
+					MultiLineString.class,
+					Polygon.class, 
+					MultiPolygon.class, 
+					GeometryCollection.class };
+  }
+  
   private Point createPointFromInternalCoord(Coordinate coord, Geometry exemplar)
   {
     exemplar.getPrecisionModel().makePrecise(coord);
