@@ -43,6 +43,11 @@ import com.vividsolutions.jts.util.Assert;
 /**
  * Supplies a set of utility methods for building Geometry objects from lists
  * of Coordinates.
+ * <p>
+ * Note that the factory constructor methods do <b>not</b> change the input coordinates in any way.
+ * In particular, they are not rounded to the supplied <tt>PrecisionModel</tt>.
+ * It is assumed that input Coordinates meet the given precision.
+ *
  *
  * @version 1.7
  */
@@ -207,36 +212,51 @@ public class GeometryFactory
   }
 
   /**
-   *  If the <code>Envelope</code> is a null <code>Envelope</code>, returns an
-   *  empty <code>Point</code>. If the <code>Envelope</code> is a point, returns
-   *  a non-empty <code>Point</code>. If the <code>Envelope</code> is a
-   *  rectangle, returns a <code>Polygon</code> whose points are (minx, miny),
-   *  (maxx, miny), (maxx, maxy), (minx, maxy), (minx, miny).
-   *
-   *@param  envelope        the <code>Envelope</code> to convert to a <code>Geometry</code>
-   *@param  precisionModel  the specification of the grid of allowable points
-   *      for the new <code>Geometry</code>
-   *@param  SRID            the ID of the Spatial Reference System used by the
-   *      <code>Envelope</code>
-   *@return                 an empty <code>Point</code> (for null <code>Envelope</code>
-   *      s), a <code>Point</code> (when min x = max x and min y = max y) or a
+   * Creates a {@link Geometry} with the same extent as the given envelope.
+   * The Geometry returned is guaranteed to be valid.  
+   * To provide this behaviour, the following cases occur:
+   * <p>
+   * If the <code>Envelope</code> is:
+   * <ul>
+   * <li>null : returns an empty {@link Point}
+   * <li>a point : returns a non-empty {@link Point}
+   * <li>a line : returns a two-point {@link LineString}
+   * <li>a rectangle : returns a {@link Polygon}> whose points are (minx, miny),
+   *  (minx, maxy), (maxx, maxy), (maxx, miny), (minx, miny).
+   * </ul>
+   * 
+   *@param  envelope the <code>Envelope</code> to convert
+   *@return an empty <code>Point</code> (for null <code>Envelope</code>s), 
+   *	a <code>Point</code> (when min x = max x and min y = max y) or a
    *      <code>Polygon</code> (in all other cases)
-   *@throws  <code>         TopologyException</code> if <code>coordinates</code>
-   *      is not a closed linestring, that is, if the first and last coordinates
-   *      are not equal
    */
-  public Geometry toGeometry(Envelope envelope) {
+  public Geometry toGeometry(Envelope envelope) 
+  {
+  	// null envelope - return empty point geometry
     if (envelope.isNull()) {
       return createPoint((CoordinateSequence)null);
     }
+    
+    // point?
     if (envelope.getMinX() == envelope.getMaxX() && envelope.getMinY() == envelope.getMaxY()) {
       return createPoint(new Coordinate(envelope.getMinX(), envelope.getMinY()));
     }
+    
+    // vertical or horizontal line?
+    if (envelope.getMinX() == envelope.getMaxX()
+    		|| envelope.getMinY() == envelope.getMaxY()) {
+    	return createLineString(new Coordinate[]{
+          new Coordinate(envelope.getMinX(), envelope.getMinY()),
+          new Coordinate(envelope.getMaxX(), envelope.getMaxY())
+          });
+    }
+
+    // create a CW ring for the polygon 
     return createPolygon(createLinearRing(new Coordinate[]{
         new Coordinate(envelope.getMinX(), envelope.getMinY()),
-        new Coordinate(envelope.getMaxX(), envelope.getMinY()),
-        new Coordinate(envelope.getMaxX(), envelope.getMaxY()),
         new Coordinate(envelope.getMinX(), envelope.getMaxY()),
+        new Coordinate(envelope.getMaxX(), envelope.getMaxY()),
+        new Coordinate(envelope.getMaxX(), envelope.getMinY()),
         new Coordinate(envelope.getMinX(), envelope.getMinY())
         }), null);
   }
@@ -320,22 +340,23 @@ public class GeometryFactory
   }
 
   /**
-   * Creates a MultiPoint using the given Points; a null or empty array will
-   * create an empty MultiPoint.
-   * @param coordinates an array without null elements, or an empty array, or null
+   * Creates a MultiPoint using the given Points.
+   * A null or empty array will create an empty MultiPoint.
+   *
+   * @param coordinates an array (without null elements), or an empty array, or <code>null</code>
+   * @return a MultiPoint object
    */
   public MultiPoint createMultiPoint(Point[] point) {
   	return new MultiPoint(point, this);
   }
 
   /**
-   * Creates a MultiPoint using the given Coordinates; a null or empty array
-   * will create an empty MultiPoint.
+   * Creates a {@link MultiPoint} using the given {@link Coordinate}s.
+   * A null or empty array will create an empty MultiPoint.
    *
-   * @param coordinates
-   *            an array without null elements, or an empty array, or null
+   * @param coordinates an array (without null elements), or an empty array, or <code>null</code>
+   * @return a MultiPoint object
    */
-
   public MultiPoint createMultiPoint(Coordinate[] coordinates) {
       return createMultiPoint(coordinates != null
                               ? getCoordinateSequenceFactory().create(coordinates)
@@ -343,9 +364,11 @@ public class GeometryFactory
   }
 
   /**
-   * Creates a MultiPoint using the given CoordinateSequence; a null or empty CoordinateSequence will
-   * create an empty MultiPoint.
-   * @param coordinates a CoordinateSequence possibly empty, or null
+   * Creates a MultiPoint using the given CoordinateSequence.
+   * A a null or empty CoordinateSequence will create an empty MultiPoint.
+   *
+   * @param coordinates a CoordinateSequence (possibly empty), or <code>null</code>
+   * @return a MultiPoint object
    */
   public MultiPoint createMultiPoint(CoordinateSequence coordinates) {
     if (coordinates == null) {
@@ -358,8 +381,6 @@ public class GeometryFactory
     return createMultiPoint(points);
   }
 
-
-
   /**
    * Constructs a <code>Polygon</code> with the given exterior boundary and
    * interior boundaries.
@@ -405,8 +426,13 @@ public class GeometryFactory
    *      .
    */
   public Geometry buildGeometry(Collection geomList) {
+  	
+  	/**
+  	 * Determine some facts about the geometries in the list
+  	 */
     Class geomClass = null;
     boolean isHeterogeneous = false;
+    boolean hasGeometryCollection = false;
     for (Iterator i = geomList.iterator(); i.hasNext(); ) {
       Geometry geom = (Geometry) i.next();
       Class partClass = geom.getClass();
@@ -416,12 +442,18 @@ public class GeometryFactory
       if (partClass != geomClass) {
         isHeterogeneous = true;
       }
+      if (geom instanceof GeometryCollection)
+        hasGeometryCollection = true;
     }
+    
+    /**
+     * Now construct an appropriate geometry to return
+     */
     // for the empty geometry, return an empty GeometryCollection
     if (geomClass == null) {
       return createGeometryCollection(null);
     }
-    if (isHeterogeneous) {
+    if (isHeterogeneous || hasGeometryCollection) {
       return createGeometryCollection(toGeometryArray(geomList));
     }
     // at this point we know the collection is hetereogenous.
@@ -439,7 +471,7 @@ public class GeometryFactory
       else if (geom0 instanceof Point) {
         return createMultiPoint(toPointArray(geomList));
       }
-      Assert.shouldNeverReachHere();
+      Assert.shouldNeverReachHere("Unhandled class: " + geom0.getClass().getName());
     }
     return geom0;
   }
