@@ -35,9 +35,11 @@
 package com.vividsolutions.jts.geom;
 
 /**
- *  A Dimensionally Extended Nine-Intersection Model (DE-9IM) matrix. This class
- *  can used to represent both computed DE-9IM's (like 212FF1FF2) as well as
- *  patterns for matching them (like T*T******). <P>
+ * Models a Dimensionally Extended Nine-Intersection Model (DE-9IM) matrix. 
+ * This class is used to represent intersection matrices (such as "212FF1FF2")
+ * capturing the topological relationship between two {@link Geometry}s. 
+ * It can also be represent patterns (such as "T*T******")for matching 
+ * existing matrices.
  *
  *  Methods are provided to:
  *  <UL>
@@ -50,7 +52,13 @@ package com.vividsolutions.jts.geom;
  *
  *  For a description of the DE-9IM, see the <A
  *  HREF="http://www.opengis.org/techno/specs.htm">OpenGIS Simple Features
- *  Specification for SQL</A> .
+ *  Specification for SQL</A>.
+ *  
+ * The entries of the matrix are defined by the constants in the {@link Dimension} class.
+ * The indices of the matrix represent the topological locations 
+ * that occur in a geometry (Interior, Boundary, Exterior).  
+ * These are provided as constants in the {@link Location} class.
+ *  
  *
  *@version 1.7
  */
@@ -265,8 +273,12 @@ public class IntersectionMatrix implements Cloneable {
   }
 
   /**
-   *  Returns the value of one of this <code>IntersectionMatrix</code>s
-   *  elements.
+   *  Returns the value of one of this matrix
+   *  entries.
+   *  The value of the provided index is one of the 
+   *  values from the {@link Location} class.  
+   *  The value returned is a constant 
+   *  from the {@link Dimension} class.
    *
    *@param  row     the row of this <code>IntersectionMatrix</code>, indicating
    *      the interior, boundary or exterior of the first <code>Geometry</code>
@@ -300,7 +312,7 @@ public class IntersectionMatrix implements Cloneable {
    *      this <code>IntersectionMatrix</code> intersect
    */
   public boolean isIntersects() {
-    return !isDisjoint();
+    return ! isDisjoint();
   }
 
   /**
@@ -332,7 +344,7 @@ public class IntersectionMatrix implements Cloneable {
   }
 
   /**
-   * Returns <code>true</code> if this geometry crosses the
+   * Tests whether this geometry crosses the
    * specified geometry.
    * <p>
    * The <code>crosses</code> predicate has the following equivalent definitions:
@@ -376,7 +388,7 @@ public class IntersectionMatrix implements Cloneable {
   }
 
   /**
-   *  Returns <code>true</code> if this <code>IntersectionMatrix</code> is
+   *  Tests whether this <code>IntersectionMatrix</code> is
    *  T*F**F***.
    *
    *@return    <code>true</code> if the first <code>Geometry</code> is within
@@ -389,7 +401,7 @@ public class IntersectionMatrix implements Cloneable {
   }
 
   /**
-   *  Returns <code>true</code> if this <code>IntersectionMatrix</code> is
+   *  Tests whether this <code>IntersectionMatrix</code> is
    *  T*****FF*.
    *
    *@return    <code>true</code> if the first <code>Geometry</code> contains the
@@ -430,8 +442,8 @@ public class IntersectionMatrix implements Cloneable {
    * or <code>**FT*F***</code>
    * or <code>**F*TF***</code>
    *
-   *@return    <code>true</code> if the first <code>Geometry</code> covers the
-   *      second
+   *@return    <code>true</code> if the first <code>Geometry</code>
+   * is covered by the second
    */
   public boolean isCoveredBy() {
     boolean hasPointInCommon =
@@ -472,13 +484,12 @@ public class IntersectionMatrix implements Cloneable {
    *  <UL>
    *    <LI> T*T***T** (for two points or two surfaces)
    *    <LI> 1*T***T** (for two curves)
-   *  </UL>
-   *  .
+   *  </UL>.
    *
    *@param  dimensionOfGeometryA  the dimension of the first <code>Geometry</code>
    *@param  dimensionOfGeometryB  the dimension of the second <code>Geometry</code>
-   *@return                       <code>true</code> if the two <code>Geometry</code>
-   *      s related by this <code>IntersectionMatrix</code> overlap. For this
+   *@return                       <code>true</code> if the two <code>Geometry</code>s
+   *      related by this <code>IntersectionMatrix</code> overlap. For this
    *      function to return <code>true</code>, the <code>Geometry</code>s must
    *      be two points, two curves or two surfaces.
    */
