@@ -1,5 +1,3 @@
-
-
 /*
  * The JTS Topology Suite is a collection of Java classes that
  * implement the fundamental operations required to validate a given
@@ -34,16 +32,17 @@
  */
 package com.vividsolutions.jts.geom;
 
-import com.vividsolutions.jts.geomgraph.GeometryGraph;
-import com.vividsolutions.jts.operation.IsSimpleOp;
-
+import com.vividsolutions.jts.operation.BoundaryOp;
 
 /**
  *  Basic implementation of <code>MultiLineString</code>.
  *
  *@version 1.7
  */
-public class MultiLineString extends GeometryCollection {
+public class MultiLineString 
+	extends GeometryCollection
+	implements Lineal
+	{
   private static final long serialVersionUID = 8166665132445433741L;
   /**
    *  Constructs a <code>MultiLineString</code>.
@@ -102,18 +101,16 @@ public class MultiLineString extends GeometryCollection {
     return true;
   }
 
-  public boolean isSimple()
+  /**
+   * Gets the boundary of this geometry.
+   * The boundary of a lineal geometry is always a zero-dimensional geometry (which may be empty).
+   *
+   * @return the boundary geometry
+   * @see Geometry#getBoundary
+   */
+  public Geometry getBoundary()
   {
-    return (new IsSimpleOp()).isSimple(this);
-  }
-
-  public Geometry getBoundary() {
-    if (isEmpty()) {
-      return getFactory().createGeometryCollection(null);
-    }
-    GeometryGraph g = new GeometryGraph(0, this);
-    Coordinate[] pts = g.getBoundaryPoints();
-    return getFactory().createMultiPoint(pts);
+    return (new BoundaryOp(this)).getBoundary();
   }
 
   /**
@@ -125,12 +122,12 @@ public class MultiLineString extends GeometryCollection {
    *
    * @return a {@link MultiLineString} in the reverse order
    */
-  public MultiLineString reverse()
+  public Geometry reverse()
   {
     int nLines = geometries.length;
     LineString[] revLines = new LineString[nLines];
     for (int i = 0; i < geometries.length; i++) {
-      revLines[nLines - 1 - i] = ((LineString) geometries[i]).reverse();
+      revLines[nLines - 1 - i] = (LineString)geometries[i].reverse();
     }
     return getFactory().createMultiLineString(revLines);
   }
