@@ -41,7 +41,10 @@ import java.util.ArrayList;
  *
  *@version 1.7
  */
-public class MultiPolygon extends GeometryCollection {
+public class MultiPolygon 
+	extends GeometryCollection 
+	implements Polygonal
+{
   private static final long serialVersionUID = -551033529766975875L;
   /**
    *  Constructs a <code>MultiPolygon</code>.
@@ -93,9 +96,15 @@ public class MultiPolygon extends GeometryCollection {
     return true;
   }
 
+  /**
+   * Computes the boundary of this geometry
+   *
+   * @return a lineal geometry (which may be empty)
+   * @see Geometry#getBoundary
+   */
   public Geometry getBoundary() {
     if (isEmpty()) {
-      return getFactory().createGeometryCollection(null);
+      return getFactory().createMultiLineString(null);
     }
     ArrayList allRings = new ArrayList();
     for (int i = 0; i < geometries.length; i++) {
@@ -115,6 +124,23 @@ public class MultiPolygon extends GeometryCollection {
     }
     return super.equalsExact(other, tolerance);
   }
+  
+  /**
+   * Creates a {@link MultiPolygon} with
+   * every component reversed.
+   * The order of the components in the collection are not reversed.
+   *
+   * @return a MultiPolygon in the reverse order
+   */
+  public Geometry reverse()
+  {
+    int n = geometries.length;
+    Polygon[] revGeoms = new Polygon[n];
+    for (int i = 0; i < geometries.length; i++) {
+      revGeoms[i] = (Polygon) geometries[i].reverse();
+    }
+    return getFactory().createMultiPolygon(revGeoms);
+  }
 }
 
 
