@@ -35,13 +35,17 @@
 package com.vividsolutions.jts.geom;
 
 import com.vividsolutions.jts.util.Assert;
+import com.vividsolutions.jts.operation.valid.*;
 
 /**
  *  Basic implementation of <code>Point</code>.
  *
  *@version 1.7
  */
-public class Point extends Geometry {
+public class Point 
+	extends Geometry
+	implements Puntal
+{
   private static final long serialVersionUID = 4902022702746614570L;
   /**
    *  The <code>Coordinate</code> wrapped by this <code>Point</code>.
@@ -101,7 +105,17 @@ public class Point extends Geometry {
     return true;
   }
 
+  /**
+   * A Point is valid iff:
+   * <ul>
+   * <li>the coordinate which defines it is a valid coordinate (i.e does not have an NaN X or Y ordinate)
+   * </ul>
+   * 
+   * @return true iff the Point is valid
+   */
   public boolean isValid() {
+  	if (! IsValidOp.isValid(getCoordinate()))
+  		return false;
     return true;
   }
 
@@ -135,6 +149,14 @@ public class Point extends Geometry {
     return "Point";
   }
 
+  /**
+   * Gets the boundary of this geometry.
+   * Zero-dimensional geometries have no boundary by definition,
+   * so an empty GeometryCollection is returned.
+   *
+   * @return an empty GeometryCollection
+   * @see Geometry#getBoundary
+   */
   public Geometry getBoundary() {
     return getFactory().createGeometryCollection(null);
   }
@@ -159,9 +181,18 @@ public class Point extends Geometry {
   }
 
   public void apply(CoordinateFilter filter) {
-    if (isEmpty()) { return; }
-    filter.filter(getCoordinate());
-  }
+	    if (isEmpty()) { return; }
+	    filter.filter(getCoordinate());
+	  }
+
+  public void apply(CoordinateSequenceFilter filter) 
+  {
+	    if (isEmpty())
+        return;
+	    filter.filter(coordinates, 0);
+      if (filter.isGeometryChanged())
+        geometryChanged();
+	  }
 
   public void apply(GeometryFilter filter) {
     filter.filter(this);
@@ -171,13 +202,27 @@ public class Point extends Geometry {
     filter.filter(this);
   }
 
+  /**
+   * Creates and returns a full copy of this {@link Point} object.
+   * (including all coordinates contained by it).
+   *
+   * @return a clone of this instance
+   */
   public Object clone() {
     Point p = (Point) super.clone();
     p.coordinates = (CoordinateSequence) coordinates.clone();
     return p;// return the clone
   }
 
-  public void normalize() { }
+  public Geometry reverse()
+  {
+    return (Geometry) clone();
+  }
+  
+  public void normalize() 
+  { 
+    // a Point is always in normalized form 
+  }
 
   protected int compareToSameClass(Object other) {
     Point point = (Point) other;
