@@ -51,7 +51,10 @@ import com.vividsolutions.jts.algorithm.*;
  *
  *@version 1.7
  */
-public class Polygon extends Geometry {
+public class Polygon 
+	extends Geometry
+	implements Polygonal
+{
   private static final long serialVersionUID = -3494792200821764533L;
 
   /**
@@ -178,6 +181,12 @@ public class Polygon extends Geometry {
     return shell.isEmpty();
   }
 
+  /**
+   * Tests if a valid polygon is simple.
+   * This method always returns true, since a valid polygon is always simple
+   *
+   * @return <code>true</code>
+   */
   public boolean isSimple() {
     return true;
   }
@@ -195,7 +204,7 @@ public class Polygon extends Geometry {
     for (int i = 0; i < 5; i++) {
       double x = seq.getX(i);
       if (! (x == env.getMinX() || x == env.getMaxX())) return false;
-      double y = seq.getX(i);
+      double y = seq.getY(i);
       if (! (y == env.getMinY() || y == env.getMaxY())) return false;
     }
 
@@ -239,9 +248,9 @@ public class Polygon extends Geometry {
   public double getArea()
   {
     double area = 0.0;
-    area += Math.abs(CGAlgorithms.signedArea(shell.getCoordinates()));
+    area += Math.abs(CGAlgorithms.signedArea(shell.getCoordinateSequence()));
     for (int i = 0; i < holes.length; i++) {
-      area -= Math.abs(CGAlgorithms.signedArea(holes[i].getCoordinates()));
+      area -= Math.abs(CGAlgorithms.signedArea(holes[i].getCoordinateSequence()));
     }
     return area;
   }
@@ -261,15 +270,22 @@ public class Polygon extends Geometry {
     return len;
   }
 
+  /**
+   * Computes the boundary of this geometry
+   *
+   * @return a lineal geometry (which may be empty)
+   * @see Geometry#getBoundary
+   */
   public Geometry getBoundary() {
     if (isEmpty()) {
-      return getFactory().createGeometryCollection(null);
+      return getFactory().createMultiLineString(null);
     }
     LinearRing[] rings = new LinearRing[holes.length + 1];
     rings[0] = shell;
     for (int i = 0; i < holes.length; i++) {
       rings[i + 1] = holes[i];
     }
+    // create LineString or MultiLineString as appropriate
     if (rings.length <= 1)
       return getFactory().createLinearRing(rings[0].getCoordinateSequence());
     return getFactory().createMultiLineString(rings);
@@ -304,11 +320,25 @@ public class Polygon extends Geometry {
   }
 
   public void apply(CoordinateFilter filter) {
-    shell.apply(filter);
-    for (int i = 0; i < holes.length; i++) {
-      holes[i].apply(filter);
-    }
-  }
+	    shell.apply(filter);
+	    for (int i = 0; i < holes.length; i++) {
+	      holes[i].apply(filter);
+	    }
+	  }
+
+  public void apply(CoordinateSequenceFilter filter) 
+  {
+	    shell.apply(filter);
+      if (! filter.isDone()) {
+        for (int i = 0; i < holes.length; i++) {
+          holes[i].apply(filter);
+          if (filter.isDone()) 
+            break;
+        }
+      }
+      if (filter.isGeometryChanged())
+        geometryChanged();
+	  }
 
   public void apply(GeometryFilter filter) {
     filter.filter(this);
@@ -322,6 +352,12 @@ public class Polygon extends Geometry {
     }
   }
 
+  /**
+   * Creates and returns a full copy of this {@link Polygon} object.
+   * (including all coordinates contained by it).
+   *
+   * @return a clone of this instance
+   */
   public Object clone() {
     Polygon poly = (Polygon) super.clone();
     poly.shell = (LinearRing) shell.clone();
@@ -388,5 +424,15 @@ public class Polygon extends Geometry {
     }
   }
 
+  public Geometry reverse()
+  {
+    Polygon poly = (Polygon) super.clone();
+    poly.shell = (LinearRing) ((LinearRing) shell.clone()).reverse();
+    poly.holes = new LinearRing[holes.length];
+    for (int i = 0; i < holes.length; i++) {
+      poly.holes[i] = (LinearRing) ((LinearRing) holes[i].clone()).reverse();
+    }
+    return poly;// return the clone
+  }
 }
 
