@@ -64,7 +64,7 @@ import java.util.Map;
  * <li>FLOATING_SINGLE - represents single precision floating point.
  * <li>FIXED - represents a model with a fixed number of decimal places.
  *  A Fixed Precision Model is specified by a scale factor.
- *  The scale factor specifies the grid which numbers are rounded to.
+ *  The scale factor specifies the size of the grid which numbers are rounded to.
  *  Input coordinates are mapped to fixed coordinates according to the following
  *  equations:
  *    <UL>
@@ -72,6 +72,10 @@ import java.util.Map;
  *      <LI> jtsPt.y = round( (inputPt.y * scale ) / scale
  *    </UL>
  * </ul>
+ * For example, to specify 3 decimal places of precision, use a scale factor
+ * of 1000. To specify -3 decimal places of precision (i.e. rounding to
+ * the nearest 1000), use a scale factor of 0.001.
+ * <p>
  *  Coordinates are represented internally as Java double-precision values.
  * Since Java uses the IEEE-394 floating point standard, this
  *  provides 53 bits of precision. (Thus the maximum precisely representable
@@ -83,20 +87,27 @@ import java.util.Map;
  */
 public class PrecisionModel implements Serializable, Comparable
 {
+	/**
+	 * Determines which of two {@link PrecisionModel}s is the most precise
+	 * (allows the greatest number of significant digits).
+	 * 
+	 * @param pm1 a PrecisionModel
+	 * @param pm2 a PrecisionModel
+	 * @return the PrecisionModel which is most precise
+	 */
+	public static PrecisionModel mostPrecise(PrecisionModel pm1, PrecisionModel pm2)
+	{
+		if (pm1.compareTo(pm2) >= 0)
+			return pm1;
+		return pm2;
+	}
+	
   private static final long serialVersionUID = 7777263578777803835L;
 
   /**
    * The types of Precision Model which JTS supports.
-   * <p>
-   * This class is only for use to support the "enums" for the types of precision model.
-   * <p>
-   * <i>
-   * Note: Type should be declared as private to this class,
-   * but JBuilder 7 throws a compiler exception
-   * when trying to compile with the "private" keyword.  Package-private is safe enough.
-   * </i>
    */
-  static class Type
+  public static class Type
       implements Serializable
   {
     private static final long serialVersionUID = -5528602631731589822L;
@@ -228,33 +239,36 @@ public class PrecisionModel implements Serializable, Comparable
    *
    * @return the maximum number of decimal places provided by this precision model
    */
-  	public int getMaximumSignificantDigits() {
-		int maxSigDigits = 16;
-		if (modelType == FLOATING) {
-			maxSigDigits = 16;
-		} else if (modelType == FLOATING_SINGLE) {
-			maxSigDigits = 6;
-		} else if (modelType == FIXED) {
-			maxSigDigits = 1 + (int) Math.ceil(Math.log(getScale()) / Math.log(10));
-		}
-		return maxSigDigits;
-	}
+  public int getMaximumSignificantDigits() {
+    int maxSigDigits = 16;
+    if (modelType == FLOATING) {
+      maxSigDigits = 16;
+    } else if (modelType == FLOATING_SINGLE) {
+      maxSigDigits = 6;
+    } else if (modelType == FIXED) {
+      maxSigDigits = 1 + (int) Math.ceil(Math.log(getScale()) / Math.log(10));
+    }
+    return maxSigDigits;
+  }
 
   /**
-   *  Returns the multiplying factor used to obtain a precise coordinate.
-   * This method is private because PrecisionModel is intended to
-   * be an immutable (value) type.
+   * Returns the scale factor used to specify a fixed precision model.
+   * The number of decimal places of precision is 
+   * equal to the base-10 logarithm of the scale factor.
+   * Non-integral and negative scale factors are supported.
+   * Negative scale factors indicate that the places 
+   * of precision is to the left of the decimal point.  
    *
-   *@return    the amount by which to multiply a coordinate after subtracting
-   *      the offset
+   *@return the scale factor for the fixed precision model
    */
   public double getScale() {
     return scale;
   }
 
   /**
-   * Gets the type of this PrecisionModel
-   * @return the type of this PrecisionModel
+   * Gets the type of this precision model
+   * @return the type of this precision model
+   * @see Type
    */
   public Type getType()
   {
@@ -262,9 +276,7 @@ public class PrecisionModel implements Serializable, Comparable
   }
   /**
    *  Sets the multiplying factor used to obtain a precise coordinate.
-   * This method is private because PrecisionModel is intended to
-   * be an immutable (value) type.
-   *
+   * This method is private because PrecisionModel is an immutable (value) type.
    */
   private void setScale(double scale)
   {
@@ -344,8 +356,7 @@ public class PrecisionModel implements Serializable, Comparable
   }
 
   /**
-   *  Sets <code>external</code> to the external representation of <code>internal</code>
-   *  .
+   *  Sets <code>external</code> to the external representation of <code>internal</code>.
    *
    *@param  internal  the original coordinate
    *@param  external  the coordinate whose values will be changed to the
@@ -359,14 +370,20 @@ public class PrecisionModel implements Serializable, Comparable
 
   /**
    * Rounds a numeric value to the PrecisionModel grid.
-   * Symmetric Arithmetic Rounding is used, to provide
+   * Asymmetric Arithmetic Rounding is used, to provide
    * uniform rounding behaviour no matter where the number is
    * on the number line.
    * <p>
-   * <b>Note:</b> Java's <code>Math#rint</code> uses the "banker's rounding" algorithm,
+   * This method has no effect on NaN values.
+   * <p>
+   * <b>Note:</b> Java's <code>Math#rint</code> uses the "Banker's Rounding" algorithm,
    * which is not suitable for precision operations elsewhere in JTS.
    */
-  public double makePrecise(double val) {
+  public double makePrecise(double val) 
+  {
+  	// don't change NaN values
+  	if (Double.isNaN(val)) return val;
+  	
   	if (modelType == FLOATING_SINGLE) {
   		float floatSingleVal = (float) val;
   		return (double) floatSingleVal;
