@@ -34,29 +34,34 @@
  */
 package com.vividsolutions.jts.operation;
 
-import java.io.PrintStream;
-import java.util.*;
-import com.vividsolutions.jts.util.*;
 import com.vividsolutions.jts.algorithm.*;
 import com.vividsolutions.jts.geom.*;
 import com.vividsolutions.jts.geomgraph.GeometryGraph;
 
 /**
- * The base class for operations that require {@link GeometryGraph)s.
+ * The base class for operations that require {@link GeometryGraph}s.
  *
  * @version 1.7
  */
 public class GeometryGraphOperation
 {
-  protected final CGAlgorithms cga = new CGAlgorithms();
   protected final LineIntersector li = new RobustLineIntersector();
   protected PrecisionModel resultPrecisionModel;
+
   /**
    * The operation args into an array so they can be accessed by index
    */
   protected GeometryGraph[] arg;  // the arg(s) of the operation
 
   public GeometryGraphOperation(Geometry g0, Geometry g1)
+  {
+    this(g0, g1,
+         BoundaryNodeRule.OGC_SFS_BOUNDARY_RULE
+//         BoundaryNodeRule.ENDPOINT_BOUNDARY_RULE
+         );
+  }
+
+  public GeometryGraphOperation(Geometry g0, Geometry g1, BoundaryNodeRule boundaryNodeRule)
   {
     // use the most precise model for the result
     if (g0.getPrecisionModel().compareTo(g1.getPrecisionModel()) >= 0)
@@ -65,9 +70,10 @@ public class GeometryGraphOperation
       setComputationPrecision(g1.getPrecisionModel());
 
     arg = new GeometryGraph[2];
-    arg[0] = new GeometryGraph(0, g0);
-    arg[1] = new GeometryGraph(1, g1);
+    arg[0] = new GeometryGraph(0, g0, boundaryNodeRule);
+    arg[1] = new GeometryGraph(1, g1, boundaryNodeRule);
   }
+
   public GeometryGraphOperation(Geometry g0) {
     setComputationPrecision(g0.getPrecisionModel());
 
