@@ -42,7 +42,7 @@ import java.util.*;
 import com.vividsolutions.jts.geom.*;
 import com.vividsolutions.jts.algorithm.*;
 import com.vividsolutions.jts.geomgraph.*;
-import com.vividsolutions.jts.noding.SegmentString;
+import com.vividsolutions.jts.noding.*;
 
 /**
  * Creates all the raw offset curves for a buffer of a {@link Geometry}.
@@ -52,7 +52,6 @@ import com.vividsolutions.jts.noding.SegmentString;
  */
 public class OffsetCurveSetBuilder {
 
-  private CGAlgorithms cga = new RobustCGAlgorithms();
 
   private Geometry inputGeom;
   private double distance;
@@ -105,7 +104,7 @@ public class OffsetCurveSetBuilder {
     // don't add null curves!
     if (coord.length < 2) return;
     // add the edge for a coordinate list which is a raw offset curve
-    SegmentString e = new SegmentString(coord,
+    SegmentString e = new NodedSegmentString(coord,
                         new Label(0, Location.BOUNDARY, leftLoc, rightLoc));
     curveList.add(e);
   }
@@ -165,6 +164,9 @@ public class OffsetCurveSetBuilder {
     // if the polygon would be completely eroded
     if (distance < 0.0 && isErodedCompletely(shell, distance))
         return;
+    // don't attemtp to buffer a polygon with too few distinct vertices
+    if (distance <= 0.0 && shellCoord.length < 3)
+    	return;
 
     addPolygonRing(
             shellCoord,
@@ -212,7 +214,7 @@ public class OffsetCurveSetBuilder {
     //Coordinate[] coord = CoordinateArrays.removeRepeatedPoints(lr.getCoordinates());
     int leftLoc  = cwLeftLoc;
     int rightLoc = cwRightLoc;
-    if (cga.isCCW(coord)) {
+    if (CGAlgorithms.isCCW(coord)) {
       leftLoc = cwRightLoc;
       rightLoc = cwLeftLoc;
       side = Position.opposite(side);
@@ -294,7 +296,7 @@ public class OffsetCurveSetBuilder {
   {
     Triangle tri = new Triangle(triangleCoord[0], triangleCoord[1], triangleCoord[2]);
     Coordinate inCentre = tri.inCentre();
-    double distToCentre = cga.distancePointLine(inCentre, tri.p0, tri.p1);
+    double distToCentre = CGAlgorithms.distancePointLine(inCentre, tri.p0, tri.p1);
     return distToCentre < Math.abs(bufferDistance);
   }
 
