@@ -50,7 +50,7 @@ public class SubgraphDepthLocater
 {
   private Collection subgraphs;
   private LineSegment seg = new LineSegment();
-  private CGAlgorithms cga = new RobustCGAlgorithms();
+  private CGAlgorithms cga = new CGAlgorithms();
 
   public SubgraphDepthLocater(List subgraphs)
   {
@@ -150,7 +150,7 @@ public class SubgraphDepthLocater
         continue;
 
       // skip if stabbing ray is right of the segment
-      if (cga.computeOrientation(seg.p0, seg.p1, stabbingRayLeftPt)
+      if (CGAlgorithms.computeOrientation(seg.p0, seg.p1, stabbingRayLeftPt)
           == CGAlgorithms.RIGHT)
         continue;
 
