@@ -38,25 +38,27 @@ import com.vividsolutions.jts.geom.util.*;
 import com.vividsolutions.jts.algorithm.*;
 
 /**
- * Computes the distance and
- * closest points between two {@link Geometry}s.
+ * Find two points on two {@link Geometry}s which lie
+ * within a given distance, or else are the nearest points
+ * on the geometries (in which case this also
+ * provides the distance between the geometries).
  * <p>
- * The distance computation finds a pair of points in the input geometries
- * which have minimum distance between them.  These points may
- * not be vertices of the geometries, but may lie in the interior of
- * a line segment. In this case the coordinate computed is a close
+ * The distance computation also finds a pair of points in the input geometries
+ * which have the minimum distance between them.
+ * If a point lies in the interior of a line segment,
+ * the coordinate computed is a close
  * approximation to the exact point.
  * <p>
  * The algorithms used are straightforward O(n^2)
  * comparisons.  This worst-case performance could be improved on
- * by using Voronoi techniques.
+ * by using Voronoi techniques or spatial indexes.
  *
  * @version 1.7
  */
-public class DistanceOp {
-
+public class DistanceOp
+{
   /**
-   * Compute the distance between the closest points of two geometries.
+   * Compute the distance between the nearest points of two geometries.
    * @param g0 a {@link Geometry}
    * @param g1 another {@link Geometry}
    * @return the distance between the geometries
@@ -67,6 +69,33 @@ public class DistanceOp {
     return distOp.distance();
   }
 
+  /**
+   * Test whether two geometries lie within a given distance of each other.
+   * @param g0 a {@link Geometry}
+   * @param g1 another {@link Geometry}
+   * @param distance the distance to test
+   * @return true if g0.distance(g1) <= distance
+   */
+  public static boolean isWithinDistance(Geometry g0, Geometry g1, double distance)
+  {
+    DistanceOp distOp = new DistanceOp(g0, g1, distance);
+    return distOp.distance() <= distance;
+  }
+
+  /**
+   * Compute the the nearest points of two geometries.
+   * The points are presented in the same order as the input Geometries.
+   *
+   * @param g0 a {@link Geometry}
+   * @param g1 another {@link Geometry}
+   * @return the nearest points in the geometries
+   */
+  public static Coordinate[] nearestPoints(Geometry g0, Geometry g1)
+  {
+    DistanceOp distOp = new DistanceOp(g0, g1);
+    return distOp.nearestPoints();
+  }
+
   /**
    * Compute the the closest points of two geometries.
    * The points are presented in the same order as the input Geometries.
@@ -74,72 +103,112 @@ public class DistanceOp {
    * @param g0 a {@link Geometry}
    * @param g1 another {@link Geometry}
    * @return the closest points in the geometries
+   * @deprecated renamed to nearestPoints
    */
   public static Coordinate[] closestPoints(Geometry g0, Geometry g1)
   {
     DistanceOp distOp = new DistanceOp(g0, g1);
-    return distOp.closestPoints();
+    return distOp.nearestPoints();
   }
 
-  private PointLocator ptLocator = new PointLocator();
+  // input
   private Geometry[] geom;
+  private double terminateDistance = 0.0;
+  // working
+  private PointLocator ptLocator = new PointLocator();
   private GeometryLocation[] minDistanceLocation;
   private double minDistance = Double.MAX_VALUE;
 
   /**
-   * Constructs a DistanceOp that computes the distance and closest points between
+   * Constructs a DistanceOp that computes the distance and nearest points between
    * the two specified geometries.
+   * @param g0 a Geometry
+   * @param g1 a Geometry
    */
   public DistanceOp(Geometry g0, Geometry g1)
+  {
+    this(g0, g1, 0.0);
+  }
+
+  /**
+   * Constructs a DistanceOp that computes the distance and nearest points between
+   * the two specified geometries.
+   * @param g0 a Geometry
+   * @param g1 a Geometry
+   * @param terminateDistance the distance on which to terminate the search
+   */
+  public DistanceOp(Geometry g0, Geometry g1, double terminateDistance)
   {
     this.geom = new Geometry[2];
     geom[0] = g0;
     geom[1] = g1;
+    this.terminateDistance = terminateDistance;
   }
 
   /**
-   * Report the distance between the closest points on the input geometries.
+   * Report the distance between the nearest points on the input geometries.
    *
    * @return the distance between the geometries
+   * @return 0 if either input geometry is empty
+   * @throws IllegalArgumentException if either input geometry is null
    */
   public double distance()
   {
+  	if (geom[0] == null || geom[1] == null)
+  		throw new IllegalArgumentException("null geometries are not supported");
+  	if (geom[0].isEmpty() || geom[1].isEmpty()) 
+  		return 0.0;
+  	
     computeMinDistance();
     return minDistance;
   }
 
   /**
-   * Report the coordinates of the closest points in the input geometries.
+   * Report the coordinates of the nearest points in the input geometries.
    * The points are presented in the same order as the input Geometries.
    *
-   * @return a pair of {@link Coordinate}s of the closest points
+   * @return a pair of {@link Coordinate}s of the nearest points
    */
-  public Coordinate[] closestPoints()
+  public Coordinate[] nearestPoints()
   {
     computeMinDistance();
-    Coordinate[] closestPts
+    Coordinate[] nearestPts
         = new Coordinate[] {
           minDistanceLocation[0].getCoordinate(),
           minDistanceLocation[1].getCoordinate() };
-    return closestPts;
+    return nearestPts;
+  }
+  
+  /**
+   * 
+   * @return
+   * @deprecated renamed to nearestPoints
+   */
+  public Coordinate[] closestPoints()
+  {
+    return nearestPoints();
   }
 
   /**
-   * Report the locations of the closest points in the input geometries.
+   * Report the locations of the nearest points in the input geometries.
    * The locations are presented in the same order as the input Geometries.
    *
-   * @return a pair of {@link GeometryLocation}s for the closest points
+   * @return a pair of {@link GeometryLocation}s for the nearest points
    */
-  public GeometryLocation[] closestLocations()
+  public GeometryLocation[] nearestLocations()
   {
     computeMinDistance();
     return minDistanceLocation;
   }
 
-  private void updateMinDistance(double dist)
+  /**
+   * 
+   * @return
+   * @deprecated renamed to nearestLocations
+   */
+  public GeometryLocation[] closestLocations()
   {
-    if (dist < minDistance)
-      minDistance = dist;
+    return nearestLocations();
   }
 
   private void updateMinDistance(GeometryLocation[] locGeom, boolean flip)
@@ -159,70 +228,71 @@ public class DistanceOp {
 
   private void computeMinDistance()
   {
+    // only compute once!
     if (minDistanceLocation != null) return;
 
     minDistanceLocation = new GeometryLocation[2];
     computeContainmentDistance();
-    if (minDistance <= 0.0) return;
-    computeLineDistance();
+    if (minDistance <= terminateDistance) return;
+    computeFacetDistance();
   }
 
   private void computeContainmentDistance()
   {
-    List polys0 = PolygonExtracter.getPolygons(geom[0]);
-    List polys1 = PolygonExtracter.getPolygons(geom[1]);
-
     GeometryLocation[] locPtPoly = new GeometryLocation[2];
-    // test if either geometry is wholely inside the other
-    if (polys1.size() > 0) {
-      List insideLocs0 = ConnectedElementLocationFilter.getLocations(geom[0]);
-      computeInside(insideLocs0, polys1, locPtPoly);
-      if (minDistance <= 0.0) {
-        minDistanceLocation[0] = locPtPoly[0];
-        minDistanceLocation[1] = locPtPoly[1];
-        return;
-      }
-    }
-    if (polys0.size() > 0) {
-      List insideLocs1 = ConnectedElementLocationFilter.getLocations(geom[1]);
-      computeInside(insideLocs1, polys0, locPtPoly);
-      if (minDistance <= 0.0) {
-        // flip locations, since we are testing geom 1 VS geom 0
-        minDistanceLocation[0] = locPtPoly[1];
-        minDistanceLocation[1] = locPtPoly[0];
-        return;
-      }
-    }
+    // test if either geometry has a vertex inside the other
+    computeContainmentDistance(0, locPtPoly);
+    if (minDistance <= terminateDistance) return;
+    computeContainmentDistance(1, locPtPoly);
   }
-  private void computeInside(List locs, List polys, GeometryLocation[] locPtPoly)
+  
+  private void computeContainmentDistance(int polyGeomIndex, GeometryLocation[] locPtPoly)
+  {
+  	int locationsIndex = 1 - polyGeomIndex;
+    List polys = PolygonExtracter.getPolygons(geom[polyGeomIndex]);
+    if (polys.size() > 0) {
+      List insideLocs = ConnectedElementLocationFilter.getLocations(geom[locationsIndex]);
+      computeContainmentDistance(insideLocs, polys, locPtPoly);
+      if (minDistance <= terminateDistance) {
+      	// this assigment is determined by the order of the args in the computeInside call above
+        minDistanceLocation[locationsIndex] = locPtPoly[0];
+        minDistanceLocation[polyGeomIndex] 	= locPtPoly[1];
+        return;
+      }
+    }	
+  }
+  
+  private void computeContainmentDistance(List locs, List polys, GeometryLocation[] locPtPoly)
   {
     for (int i = 0; i < locs.size(); i++) {
       GeometryLocation loc = (GeometryLocation) locs.get(i);
       for (int j = 0; j < polys.size(); j++) {
-        Polygon poly = (Polygon) polys.get(j);
-        computeInside(loc, poly, locPtPoly);
-        if (minDistance <= 0.0) {
-          return;
-        }
+      	computeContainmentDistance(loc, (Polygon) polys.get(j), locPtPoly);
+        if (minDistance <= terminateDistance) return;
       }
     }
   }
 
-  private void computeInside(GeometryLocation ptLoc,
+  private void computeContainmentDistance(GeometryLocation ptLoc,
       Polygon poly,
       GeometryLocation[] locPtPoly)
   {
     Coordinate pt = ptLoc.getCoordinate();
+    // if pt is not in exterior, distance to geom is 0
     if (Location.EXTERIOR != ptLocator.locate(pt, poly)) {
       minDistance = 0.0;
       locPtPoly[0] = ptLoc;
-      GeometryLocation locPoly = new GeometryLocation(poly, pt);
-      locPtPoly[1] = locPoly;
+      locPtPoly[1] = new GeometryLocation(poly, pt);;
       return;
     }
   }
 
-  private void computeLineDistance()
+  /**
+   * Computes distance between facets (lines and points)
+   * of input geometries.
+   *
+   */
+  private void computeFacetDistance()
   {
     GeometryLocation[] locGeom = new GeometryLocation[2];
 
@@ -236,22 +306,22 @@ public class DistanceOp {
     List pts0 = PointExtracter.getPoints(geom[0]);
     List pts1 = PointExtracter.getPoints(geom[1]);
 
-    // bail whenever minDistance goes to zero, since it can't get any less
+    // exit whenever minDistance goes LE than terminateDistance
     computeMinDistanceLines(lines0, lines1, locGeom);
     updateMinDistance(locGeom, false);
-    if (minDistance <= 0.0) return;
+    if (minDistance <= terminateDistance) return;
 
     locGeom[0] = null;
     locGeom[1] = null;
     computeMinDistanceLinesPoints(lines0, pts1, locGeom);
     updateMinDistance(locGeom, false);
-    if (minDistance <= 0.0) return;
+    if (minDistance <= terminateDistance) return;
 
     locGeom[0] = null;
     locGeom[1] = null;
     computeMinDistanceLinesPoints(lines1, pts0, locGeom);
     updateMinDistance(locGeom, true);
-    if (minDistance <= 0.0) return;
+    if (minDistance <= terminateDistance) return;
 
     locGeom[0] = null;
     locGeom[1] = null;
@@ -266,7 +336,7 @@ public class DistanceOp {
       for (int j = 0; j < lines1.size(); j++) {
         LineString line1 = (LineString) lines1.get(j);
         computeMinDistance(line0, line1, locGeom);
-        if (minDistance <= 0.0) return;
+        if (minDistance <= terminateDistance) return;
       }
     }
   }
@@ -280,11 +350,10 @@ public class DistanceOp {
         double dist = pt0.getCoordinate().distance(pt1.getCoordinate());
         if (dist < minDistance) {
           minDistance = dist;
-          // this is wrong - need to determine closest points on both segments!!!
           locGeom[0] = new GeometryLocation(pt0, 0, pt0.getCoordinate());
           locGeom[1] = new GeometryLocation(pt1, 0, pt1.getCoordinate());
         }
-        if (minDistance <= 0.0) return;
+        if (minDistance <= terminateDistance) return;
       }
     }
   }
@@ -297,7 +366,7 @@ public class DistanceOp {
       for (int j = 0; j < points.size(); j++) {
         Point pt = (Point) points.get(j);
         computeMinDistance(line, pt, locGeom);
-        if (minDistance <= 0.0) return;
+        if (minDistance <= terminateDistance) return;
       }
     }
   }
@@ -324,7 +393,7 @@ public class DistanceOp {
           locGeom[0] = new GeometryLocation(line0, i, closestPt[0]);
           locGeom[1] = new GeometryLocation(line1, j, closestPt[1]);
         }
-        if (minDistance <= 0.0) return;
+        if (minDistance <= terminateDistance) return;
       }
     }
   }
@@ -348,7 +417,7 @@ public class DistanceOp {
           locGeom[0] = new GeometryLocation(line, i, segClosestPoint);
           locGeom[1] = new GeometryLocation(pt, 0, coord);
         }
-        if (minDistance <= 0.0) return;
+        if (minDistance <= terminateDistance) return;
 
     }
   }
