@@ -80,9 +80,10 @@ public class EdgeRing {
       if (tryEnv.equals(testEnv))
         continue;
 
-      testPt = ptNotInList(testRing.getCoordinates(), tryRing.getCoordinates());
+//      testPt = ptNotInList(testRing.getCoordinates(), tryRing.getCoordinates());
+      testPt = CoordinateArrays.ptNotInList(testRing.getCoordinates(), tryRing.getCoordinates());
       if (tryEnv.contains(testEnv)
-          && cga.isPointInRing(testPt, tryRing.getCoordinates()) )
+          && CGAlgorithms.isPointInRing(testPt, tryRing.getCoordinates()) )
         isContained = true;
       // check if this new containing ring is smaller than the current minimum ring
       if (isContained) {
@@ -99,14 +100,14 @@ public class EdgeRing {
    * Finds a point in a list of points which is not contained in another list of points
    * @param testPts the {@link Coordinate}s to test
    * @param pts an array of {@link Coordinate}s to test the input points against
-   * @return a {@link Coordinate} from <code>testPts</code> which is not in <code>pts</code>, '
-   * or <code>null</code>
+   * @return a {@link Coordinate} from <code>testPts</code> which is not in <code>pts</code>,
+   * @return null if there is no coordinate not in the list
    */
   public static Coordinate ptNotInList(Coordinate[] testPts, Coordinate[] pts)
   {
     for (int i = 0; i < testPts.length; i++) {
       Coordinate testPt = testPts[i];
-      if (isInList(testPt, pts))
+      if (! isInList(testPt, pts))
           return testPt;
     }
     return null;
@@ -124,12 +125,11 @@ public class EdgeRing {
   {
     for (int i = 0; i < pts.length; i++) {
         if (pt.equals(pts[i]))
-            return false;
+            return true;
     }
-    return true;
+    return false;
   }
   private GeometryFactory factory;
-  private static CGAlgorithms cga = new CGAlgorithms();
 
 
   private List deList = new ArrayList();
@@ -163,7 +163,7 @@ public class EdgeRing {
   public boolean isHole()
   {
     LinearRing ring = getRing();
-    return cga.isCCW(ring.getCoordinates());
+    return CGAlgorithms.isCCW(ring.getCoordinates());
   }
 
   /**
@@ -177,7 +177,7 @@ public class EdgeRing {
   }
 
   /**
-   * Computes the {@link Polygon formed by this ring and any contained holes.
+   * Computes the {@link Polygon} formed by this ring and any contained holes.
    *
    * @return the {@link Polygon} formed by this ring and its holes.
    */
@@ -196,7 +196,8 @@ public class EdgeRing {
 
   /**
    * Tests if the {@link LinearRing} ring formed by this edge ring is topologically valid.
-   * @return
+   * 
+   * @return true if the ring is valid
    */
   public boolean isValid()
   {
