@@ -65,8 +65,6 @@ public class DirectedEdge
     return edges;
   }
 
-  protected static final CGAlgorithms cga = new CGAlgorithms();
-
   protected Edge parentEdge;
   protected Node from;
   protected Node to;
@@ -183,7 +181,7 @@ public class DirectedEdge
    * <li>first compare the quadrants. If the quadrants are different, it it
    * trivial to determine which vector is "greater".
    * <li>if the vectors lie in the same quadrant, the robust
-   * {@link RobustCGAlgorithms#computeOrientation(Coordinate, Coordinate, Coordinate)}
+   * {@link CGAlgorithms#computeOrientation(Coordinate, Coordinate, Coordinate)}
    * function can be used to decide the relative orientation of the vectors.
    * </ul>
    */
@@ -204,7 +202,7 @@ public class DirectedEdge
    * <li>first compare the quadrants. If the quadrants are different, it it
    * trivial to determine which vector is "greater".
    * <li>if the vectors lie in the same quadrant, the robust
-   * {@link RobustCGAlgorithms#computeOrientation(Coordinate, Coordinate, Coordinate)}
+   * {@link CGAlgorithms#computeOrientation(Coordinate, Coordinate, Coordinate)}
    * function can be used to decide the relative orientation of the vectors.
    * </ul>
    */
