@@ -30,7 +30,8 @@ public class DouglasPeuckerSimplifier
 
   private Geometry inputGeom;
   private double distanceTolerance;
-
+  private boolean isEnsureValidTopology = true;
+  
   public DouglasPeuckerSimplifier(Geometry inputGeom)
   {
     this.inputGeom = inputGeom;
@@ -40,8 +41,7 @@ public class DouglasPeuckerSimplifier
    * Sets the distance tolerance for the simplification.
    * All vertices in the simplified geometry will be within this
    * distance of the original geometry.
-   * The tolerance value must be non-negative.  A tolerance value
-   * of zero is effectively a no-op.
+   * The tolerance value must be non-negative. 
    *
    * @param distanceTolerance the approximation tolerance to use
    */
@@ -51,14 +51,40 @@ public class DouglasPeuckerSimplifier
     this.distanceTolerance = distanceTolerance;
   }
 
+  /**
+   * Controls whether simplified polygons will be "fixed"
+   * to have valid topology.
+   * The caller may choose to disable this because:
+   * <ul>
+   * <li>valid topology is not required
+   * <li>fixing topology is a relative expensive operation
+   * <li>in some pathological cases the topology fixing operation may either fail or run for too long
+   * </ul>
+   * 
+   * The default is to fix polygon topology.
+   * 
+   * @param isEnsureValidTopology
+   */
+  public void setEnsureValid(boolean isEnsureValidTopology)
+  {
+  	this.isEnsureValidTopology = isEnsureValidTopology;
+  }
+  
   public Geometry getResultGeometry()
   {
-    return (new DPTransformer()).transform(inputGeom);
+    return (new DPTransformer(isEnsureValidTopology)).transform(inputGeom);
   }
 
 class DPTransformer
     extends GeometryTransformer
 {
+  private boolean isEnsureValidTopology = true;
+
+	public DPTransformer(boolean isEnsureValidTopology)
+	{
+		this.isEnsureValidTopology = isEnsureValidTopology;
+	}
+	
   protected CoordinateSequence transformCoordinates(CoordinateSequence coords, Geometry parent)
   {
     Coordinate[] inputPts = coords.toCoordinateArray();
@@ -66,18 +92,39 @@ class DPTransformer
     return factory.getCoordinateSequenceFactory().create(newPts);
   }
 
+  /**
+   * Simplifies a polygon, fixing it if required.
+   */
   protected Geometry transformPolygon(Polygon geom, Geometry parent) {
-    Geometry roughGeom = super.transformPolygon(geom, parent);
+    Geometry rawGeom = super.transformPolygon(geom, parent);
     // don't try and correct if the parent is going to do this
     if (parent instanceof MultiPolygon) {
-      return roughGeom;
+      return rawGeom;
     }
-    return createValidArea(roughGeom);
+    return createValidArea(rawGeom);
   }
 
+  /**
+   * Simplifies a LinearRing.  If the simplification results 
+   * in a degenerate ring, remove the component.
+   * 
+   * @return null if the simplification results in a degenerate ring
+   */
+  protected Geometry transformLinearRing(LinearRing geom, Geometry parent) 
+  {
+  	boolean removeDegenerateRings = parent instanceof Polygon;
+  	Geometry simpResult = super.transformLinearRing(geom, parent);
+  	if (removeDegenerateRings && ! (simpResult instanceof LinearRing))
+  		return null;;
+  	return simpResult;
+  }
+  
+  /**
+   * Simplifies a MultiPolygon, fixing it if required.
+   */
   protected Geometry transformMultiPolygon(MultiPolygon geom, Geometry parent) {
-    Geometry roughGeom = super.transformMultiPolygon(geom, parent);
-    return createValidArea(roughGeom);
+    Geometry rawGeom = super.transformMultiPolygon(geom, parent);
+    return createValidArea(rawGeom);
   }
 
   /**
@@ -90,12 +137,14 @@ class DPTransformer
    * areas.  This also may return empty geometries, if the input
    * has no actual area.
    *
-   * @param roughAreaGeom an area geometry possibly containing self-intersections
+   * @param rawAreaGeom an area geometry possibly containing self-intersections
    * @return a valid area geometry
    */
-  private Geometry createValidArea(Geometry roughAreaGeom)
+  private Geometry createValidArea(Geometry rawAreaGeom)
   {
-    return roughAreaGeom.buffer(0.0);
+  	if ( isEnsureValidTopology)
+  		return rawAreaGeom.buffer(0.0);
+  	return rawAreaGeom;
   }
 }
 
