@@ -4,9 +4,15 @@ import java.util.*;
 import com.vividsolutions.jts.geom.*;
 
 /**
+ * Represents a {@link LineString} which can be modified to a simplified shape.  
+ * Every line segment in the parent LineString is represented as a {@link TaggedLineSegment}.
+ * This class provides an attribute which specifies the minimum allowable length
+ * for the modified result.
+ * 
  * @version 1.7
  */
-public class TaggedLineString {
+public class TaggedLineString
+{
 
   private LineString parentLine;
   private TaggedLineSegment[] segs;
