@@ -56,9 +56,10 @@ import com.vividsolutions.jts.geom.*;
  */
 public class GeometricShapeFactory
 {
-  private GeometryFactory geomFact;
-  private Dimensions dim = new Dimensions();
-  private int nPts = 100;
+  protected GeometryFactory geomFact;
+  protected PrecisionModel precModel = null;
+  protected Dimensions dim = new Dimensions();
+  protected int nPts = 100;
 
   /**
    * Create a shape factory which will create shapes using the default
@@ -78,8 +79,14 @@ public class GeometricShapeFactory
   public GeometricShapeFactory(GeometryFactory geomFact)
   {
     this.geomFact = geomFact;
+    precModel = geomFact.getPrecisionModel();
   }
 
+  public void setEnvelope(Envelope env)
+  {
+  	dim.setEnvelope(env);
+  }
+  
   /**
    * Sets the location of the shape by specifying the base coordinate
    * (which in most cases is the
@@ -148,22 +155,22 @@ public class GeometricShapeFactory
     for (i = 0; i < nSide; i++) {
       double x = env.getMinX() + i * XsegLen;
       double y = env.getMinY();
-      pts[ipt++] = new Coordinate(x, y);
+      pts[ipt++] = createCoord(x, y);
     }
     for (i = 0; i < nSide; i++) {
       double x = env.getMaxX();
       double y = env.getMinY() + i * YsegLen;
-      pts[ipt++] = new Coordinate(x, y);
+      pts[ipt++] = createCoord(x, y);
     }
     for (i = 0; i < nSide; i++) {
       double x = env.getMaxX() - i * XsegLen;
       double y = env.getMaxY();
-      pts[ipt++] = new Coordinate(x, y);
+      pts[ipt++] = createCoord(x, y);
     }
     for (i = 0; i < nSide; i++) {
       double x = env.getMinX();
       double y = env.getMaxY() - i * YsegLen;
-      pts[ipt++] = new Coordinate(x, y);
+      pts[ipt++] = createCoord(x, y);
     }
     pts[ipt++] = new Coordinate(pts[0]);
 
@@ -193,10 +200,75 @@ public class GeometricShapeFactory
         double ang = i * (2 * Math.PI / nPts);
         double x = xRadius * Math.cos(ang) + centreX;
         double y = yRadius * Math.sin(ang) + centreY;
-        Coordinate pt = new Coordinate(x, y);
-        pts[iPt++] = pt;
+        pts[iPt++] = createCoord(x, y);
     }
-    pts[iPt] = pts[0];
+    pts[iPt] = new Coordinate(pts[0]);
+
+    LinearRing ring = geomFact.createLinearRing(pts);
+    Polygon poly = geomFact.createPolygon(ring, null);
+    return poly;
+  }
+  /**
+   * Creates a squircular {@link Polygon}.
+   *
+   * @return a squircle
+   */
+  public Polygon createSquircle()
+  /**
+   * Creates a squircular {@link Polygon}.
+   *
+   * @return a squircle
+   */
+  {
+  	return createSupercircle(4);
+  }
+  
+  /**
+   * Creates a supercircular {@link Polygon}
+   * of a given positive power.
+   *
+   * @return a supercircle
+   */
+  public Polygon createSupercircle(double power)
+  {
+  	double recipPow = 1.0 / power;
+  	
+    Envelope env = dim.getEnvelope();
+
+    double radius = dim.getMinSize() / 2;
+    Coordinate centre = dim.getCentre();
+    
+    double r4 = Math.pow(radius, power);
+    double y0 = radius;
+    
+    double xyInt = Math.pow(r4 / 2, recipPow);
+    
+    int nSegsInOct = nPts / 8;
+    int totPts = nSegsInOct * 8 + 1;
+    Coordinate[] pts = new Coordinate[totPts];
+    double xInc = xyInt / nSegsInOct;
+    
+    for (int i = 0; i <= nSegsInOct; i++) {
+  		double x = 0.0;
+  		double y = y0;
+    	if (i != 0) {
+    		x = xInc * i;
+    		double x4 = Math.pow(x, power);
+    		y = Math.pow(r4 - x4, recipPow);
+    	}
+      pts[i] = createCoordTrans(x, y, centre);
+      pts[2 * nSegsInOct - i] = createCoordTrans(y, x, centre);
+      
+      pts[2 * nSegsInOct + i] = createCoordTrans(y, -x, centre);
+      pts[4 * nSegsInOct - i] = createCoordTrans(x, -y, centre);
+      
+      pts[4 * nSegsInOct + i] = createCoordTrans(-x, -y, centre);
+      pts[6 * nSegsInOct - i] = createCoordTrans(-y, -x, centre);
+      
+      pts[6 * nSegsInOct + i] = createCoordTrans(-y, x, centre);
+      pts[8 * nSegsInOct - i] = createCoordTrans(-x, y, centre);
+    }
+    pts[pts.length-1] = new Coordinate(pts[0]);
 
     LinearRing ring = geomFact.createLinearRing(pts);
     Polygon poly = geomFact.createPolygon(ring, null);
@@ -204,13 +276,16 @@ public class GeometricShapeFactory
   }
 
    /**
-    * Creates a elliptical arc, as a LineString.
+    * Creates an elliptical arc, as a {@link LineString}.
+    * The arc is always created in a counter-clockwise direction.
     *
+    * @param startAng start angle in radians
+    * @param angExtent size of angle in radians
     * @return an elliptical arc
     */
   public LineString createArc(
      double startAng,
-     double endAng)
+     double angExtent)
   {
     Envelope env = dim.getEnvelope();
     double xRadius = env.getWidth() / 2.0;
@@ -219,10 +294,10 @@ public class GeometricShapeFactory
     double centreX = env.getMinX() + xRadius;
     double centreY = env.getMinY() + yRadius;
 
-     double angSize = (endAng - startAng);
+     double angSize = angExtent;
      if (angSize <= 0.0 || angSize > 2 * Math.PI)
        angSize = 2 * Math.PI;
-     double angInc = angSize / nPts;
+     double angInc = angSize / (nPts - 1);
 
      Coordinate[] pts = new Coordinate[nPts];
      int iPt = 0;
@@ -230,15 +305,66 @@ public class GeometricShapeFactory
          double ang = startAng + i * angInc;
          double x = xRadius * Math.cos(ang) + centreX;
          double y = yRadius * Math.sin(ang) + centreY;
-         Coordinate pt = new Coordinate(x, y);
-         geomFact.getPrecisionModel().makePrecise(pt);
-         pts[iPt++] = pt;
+         pts[iPt++] = createCoord(x, y);
      }
      LineString line = geomFact.createLineString(pts);
      return line;
    }
 
-  private class Dimensions
+  /**
+   * Creates an elliptical arc polygon.
+   * The polygon is formed from the specified arc of an ellipse
+   * and the two radii connecting the endpoints to the centre of the ellipse.
+   *
+   * @param startAng start angle in radians
+   * @param angExtent size of angle in radians
+   * @return an elliptical arc polygon
+   */
+  public Polygon createArcPolygon(double startAng, double angExtent) {
+    Envelope env = dim.getEnvelope();
+    double xRadius = env.getWidth() / 2.0;
+    double yRadius = env.getHeight() / 2.0;
+
+    double centreX = env.getMinX() + xRadius;
+    double centreY = env.getMinY() + yRadius;
+
+    double angSize = angExtent;
+    if (angSize <= 0.0 || angSize > 2 * Math.PI)
+      angSize = 2 * Math.PI;
+    double angInc = angSize / (nPts - 1);
+    // double check = angInc * nPts;
+    // double checkEndAng = startAng + check;
+
+    Coordinate[] pts = new Coordinate[nPts + 2];
+
+    int iPt = 0;
+    pts[iPt++] = createCoord(centreX, centreY);
+    for (int i = 0; i < nPts; i++) {
+      double ang = startAng + angInc * i;
+
+      double x = xRadius * Math.cos(ang) + centreX;
+      double y = yRadius * Math.sin(ang) + centreY;
+      pts[iPt++] = createCoord(x, y);
+    }
+    pts[iPt++] = createCoord(centreX, centreY);
+    LinearRing ring = geomFact.createLinearRing(pts);
+    Polygon geom = geomFact.createPolygon(ring, null);
+    return geom;
+  }
+
+  protected Coordinate createCoord(double x, double y)
+  {
+  	Coordinate pt = new Coordinate(x, y);
+    precModel.makePrecise(pt);
+    return pt;
+  }
+  
+  protected Coordinate createCoordTrans(double x, double y, Coordinate trans)
+  {
+  	return createCoord(x + trans.x, y + trans.y);
+  }
+  
+  protected class Dimensions
   {
     public Coordinate base;
     public Coordinate centre;
@@ -246,16 +372,35 @@ public class GeometricShapeFactory
     public double height;
 
     public void setBase(Coordinate base)  {  this.base = base;    }
+    public Coordinate getBase() { return base; }
+    
     public void setCentre(Coordinate centre)  {  this.centre = centre;    }
+    public Coordinate getCentre() { return centre; }
+   
     public void setSize(double size)
     {
       height = size;
       width = size;
     }
 
+    public double getMinSize()
+    {
+    	return Math.min(width, height);
+    }
     public void setWidth(double width) { this.width = width; }
+    public double getWidth() { return width; }
+    public double getHeight() { return height; }
+    
     public void setHeight(double height) { this.height = height; }
 
+    public void setEnvelope(Envelope env)
+    {
+    	this.width = env.getWidth();
+    	this.height = env.getHeight();
+    	this.base = new Coordinate(env.getMinX(), env.getMinY());
+    	this.centre = new Coordinate(env.centre());
+    }
+    
     public Envelope getEnvelope() {
       if (base != null) {
         return new Envelope(base.x, base.x + width, base.y, base.y + height);
@@ -266,5 +411,6 @@ public class GeometricShapeFactory
       }
       return new Envelope(0, width, 0, height);
     }
+    
   }
 }
